@@ -1,12 +1,10 @@
-import inspect
-from typing import TYPE_CHECKING, Optional, Union
+from typing import TYPE_CHECKING, Optional, Union, List, cast, Dict, Any
 
 from chromadb.api.models.CollectionCommon import CollectionCommon
 from chromadb.api.types import (
     URI,
     CollectionMetadata,
     Embedding,
-    IncludeEnum,
     PyEmbedding,
     Include,
     Metadata,
@@ -19,8 +17,12 @@ from chromadb.api.types import (
     ID,
     OneOrMany,
     WhereDocument,
-    IncludeEnum,
+    SearchResult,
+    maybe_cast_one_to_many,
 )
+from chromadb.api.collection_configuration import UpdateCollectionConfiguration
+from chromadb.execution.expression.plan import Search
+from typing import cast, List
 
 import logging
 
@@ -106,7 +108,7 @@ class Collection(CollectionCommon["ServerAPI"]):
         limit: Optional[int] = None,
         offset: Optional[int] = None,
         where_document: Optional[WhereDocument] = None,
-        include: Include = [IncludeEnum.metadatas, IncludeEnum.documents],
+        include: Include = ["metadatas", "documents"],
     ) -> GetResult:
         """Get embeddings and their associate data from the data store. If no ids or where filter is provided returns
         all embeddings up to limit starting at offset.
@@ -116,7 +118,7 @@ class Collection(CollectionCommon["ServerAPI"]):
             where: A Where type dict used to filter results by. E.g. `{"$and": [{"color" : "red"}, {"price": {"$gte": 4.20}}]}`. Optional.
             limit: The number of documents to return. Optional.
             offset: The offset to start returning results from. Useful for paging results with limit. Optional.
-            where_document: A WhereDocument type dict used to filter by the documents. E.g. `{$contains: {"text": "hello"}}`. Optional.
+            where_document: A WhereDocument type dict used to filter by the documents. E.g. `{"$contains": "hello"}`. Optional.
             include: A list of what to include in the results. Can contain `"embeddings"`, `"metadatas"`, `"documents"`. Ids are always included. Defaults to `["metadatas", "documents"]`. Optional.
 
         Returns:
@@ -136,13 +138,11 @@ class Collection(CollectionCommon["ServerAPI"]):
             where=get_request["where"],
             where_document=get_request["where_document"],
             include=get_request["include"],
-            sort=None,
             limit=limit,
             offset=offset,
             tenant=self.tenant,
             database=self.database,
         )
-
         return self._transform_get_response(
             response=get_results, include=get_request["include"]
         )
@@ -176,13 +176,14 @@ class Collection(CollectionCommon["ServerAPI"]):
         query_texts: Optional[OneOrMany[Document]] = None,
         query_images: Optional[OneOrMany[Image]] = None,
         query_uris: Optional[OneOrMany[URI]] = None,
+        ids: Optional[OneOrMany[ID]] = None,
         n_results: int = 10,
         where: Optional[Where] = None,
         where_document: Optional[WhereDocument] = None,
         include: Include = [
-            IncludeEnum.metadatas,
-            IncludeEnum.documents,
-            IncludeEnum.distances,
+            "metadatas",
+            "documents",
+            "distances",
         ],
     ) -> QueryResult:
         """Get the n_results nearest neighbor embeddings for provided query_embeddings or query_texts.
@@ -192,9 +193,10 @@ class Collection(CollectionCommon["ServerAPI"]):
             query_texts: The document texts to get the closes neighbors of. Optional.
             query_images: The images to get the closes neighbors of. Optional.
             query_uris: The URIs to be used with data loader. Optional.
+            ids: A subset of ids to search within. Optional.
             n_results: The number of neighbors to return for each query_embedding or query_texts. Optional.
             where: A Where type dict used to filter results by. E.g. `{"$and": [{"color" : "red"}, {"price": {"$gte": 4.20}}]}`. Optional.
-            where_document: A WhereDocument type dict used to filter by the documents. E.g. `{$contains: {"text": "hello"}}`. Optional.
+            where_document: A WhereDocument type dict used to filter by the documents. E.g. `{"$contains": "hello"}`. Optional.
             include: A list of what to include in the results. Can contain `"embeddings"`, `"metadatas"`, `"documents"`, `"distances"`. Ids are always included. Defaults to `["metadatas", "documents", "distances"]`. Optional.
 
         Returns:
@@ -213,6 +215,7 @@ class Collection(CollectionCommon["ServerAPI"]):
             query_texts=query_texts,
             query_images=query_images,
             query_uris=query_uris,
+            ids=ids,
             n_results=n_results,
             where=where,
             where_document=where_document,
@@ -221,6 +224,7 @@ class Collection(CollectionCommon["ServerAPI"]):
 
         query_results = self._client._query(
             collection_id=self.id,
+            ids=query_request["ids"],
             query_embeddings=query_request["embeddings"],
             n_results=query_request["n_results"],
             where=query_request["where"],
@@ -235,7 +239,10 @@ class Collection(CollectionCommon["ServerAPI"]):
         )
 
     def modify(
-        self, name: Optional[str] = None, metadata: Optional[CollectionMetadata] = None
+        self,
+        name: Optional[str] = None,
+        metadata: Optional[CollectionMetadata] = None,
+        configuration: Optional[UpdateCollectionConfiguration] = None,
     ) -> None:
         """Modify the collection name or metadata
 
@@ -256,11 +263,111 @@ class Collection(CollectionCommon["ServerAPI"]):
             id=self.id,
             new_name=name,
             new_metadata=metadata,
+            new_configuration=configuration,
             tenant=self.tenant,
             database=self.database,
         )
 
-        self._update_model_after_modify_success(name, metadata)
+        self._update_model_after_modify_success(name, metadata, configuration)
+
+    def fork(
+        self,
+        new_name: str,
+    ) -> "Collection":
+        """Fork the current collection under a new name. The returning collection should contain identical data to the current collection.
+        This is an experimental API that only works for Hosted Chroma for now.
+
+        Args:
+            new_name: The name of the new collection.
+
+        Returns:
+            Collection: A new collection with the specified name and containing identical data to the current collection.
+        """
+        model = self._client._fork(
+            collection_id=self.id,
+            new_name=new_name,
+            tenant=self.tenant,
+            database=self.database,
+        )
+        return Collection(
+            client=self._client,
+            model=model,
+            embedding_function=self._embedding_function,
+            data_loader=self._data_loader,
+        )
+
+    def search(
+        self,
+        searches: OneOrMany[Search],
+    ) -> SearchResult:
+        """Perform hybrid search on the collection.
+        This is an experimental API that only works for Hosted Chroma for now.
+
+        Args:
+            searches: A single Search object or a list of Search objects, each containing:
+                - where: Where expression for filtering
+                - rank: Ranking expression for hybrid search (defaults to Val(0.0))
+                - limit: Limit configuration for pagination (defaults to no limit)
+                - select: Select configuration for keys to return (defaults to empty)
+
+        Returns:
+            SearchResult: Column-major format response with:
+                - ids: List of result IDs for each search payload
+                - documents: Optional documents for each payload
+                - embeddings: Optional embeddings for each payload
+                - metadatas: Optional metadata for each payload
+                - scores: Optional scores for each payload
+                - select: List of selected keys for each payload
+
+        Raises:
+            NotImplementedError: For local/segment API implementations
+
+        Examples:
+            # Using builder pattern with Key constants
+            from chromadb.execution.expression import (
+                Search, Key, K, Knn, Val
+            )
+
+            # Note: K is an alias for Key, so K.DOCUMENT == Key.DOCUMENT
+            search = (Search()
+                .where((K("category") == "science") & (K("score") > 0.5))
+                .rank(Knn(query=[0.1, 0.2, 0.3]) * 0.8 + Val(0.5) * 0.2)
+                .limit(10, offset=0)
+                .select(K.DOCUMENT, K.SCORE, "title"))
+
+            # Direct construction
+            from chromadb.execution.expression import (
+                Search, Eq, And, Gt, Knn, Limit, Select, Key
+            )
+
+            search = Search(
+                where=And([Eq("category", "science"), Gt("score", 0.5)]),
+                rank=Knn(query=[0.1, 0.2, 0.3]),
+                limit=Limit(offset=0, limit=10),
+                select=Select(keys={Key.DOCUMENT, Key.SCORE, "title"})
+            )
+
+            # Single search
+            result = collection.search(search)
+
+            # Multiple searches at once
+            searches = [
+                Search().where(K("type") == "article").rank(Knn(query=[0.1, 0.2])),
+                Search().where(K("type") == "paper").rank(Knn(query=[0.3, 0.4]))
+            ]
+            results = collection.search(searches)
+        """
+        # Convert single search to list for consistent handling
+        searches_list = maybe_cast_one_to_many(searches)
+        if searches_list is None:
+            searches_list = []
+
+        return self._client._search(
+            collection_id=self.id,
+            searches=cast(List[Search], searches_list),
+            tenant=self.tenant,
+            database=self.database,
+        )
 
     def update(
         self,
@@ -363,7 +470,7 @@ class Collection(CollectionCommon["ServerAPI"]):
         Args:
             ids: The ids of the embeddings to delete
             where: A Where type dict used to filter the delection by. E.g. `{"$and": [{"color" : "red"}, {"price": {"$gte": 4.20}]}}`. Optional.
-            where_document: A WhereDocument type dict used to filter the deletion by the document content. E.g. `{$contains: {"text": "hello"}}`. Optional.
+            where_document: A WhereDocument type dict used to filter the deletion by the document content. E.g. `{"$contains": "hello"}`. Optional.
 
         Returns:
             None
@@ -384,41 +491,63 @@ class Collection(CollectionCommon["ServerAPI"]):
             database=self.database,
         )
 
+    def create_task(
+        self,
+        task_name: str,
+        operator_name: str,
+        output_collection_name: str,
+        params: Optional[Dict[str, Any]] = None,
+    ) -> tuple[bool, str]:
+        """Create a recurring task that processes this collection.
 
-class CollectionName(str):
-    """
-    A string wrapper to supply users with indicative message about list_collections only
-    returning collection names, in lieu of Collection object.
+        Args:
+            task_name: Unique name for this task instance
+            operator_name: Built-in operator name (e.g., "record_counter")
+            output_collection_name: Name of the collection where task output will be stored
+            params: Optional dictionary with operator-specific parameters
 
-    When a user will try to access an attribute on a CollectionName string, the __getattribute__ method
-    of str is invoked first. If a valid str method or property is found, it will be used. Otherwise, the fallback
-    __getattr__ defined here is invoked next. It will error if the requested attribute is a Collection
-    method or property.
+        Returns:
+            tuple: (success: bool, task_id: str)
 
-    For example:
-    collection_name = client.list_collections()[0] # collection_name = "test"
+        Example:
+            >>> success, task_id = collection.create_task(
+            ...     task_name="count_docs",
+            ...     operator_name="record_counter",
+            ...     output_collection_name="doc_counts",
+            ...     params={"threshold": 100}
+            ... )
+        """
+        return self._client.create_task(
+            task_name=task_name,
+            operator_name=operator_name,
+            input_collection_id=self.id,
+            output_collection_name=output_collection_name,
+            params=params,
+            tenant=self.tenant,
+            database=self.database,
+        )
 
-    collection_name.startsWith("t") # Evaluates to True.
-    # __getattribute__ is invoked first, selecting startsWith from str.
+    def remove_task(
+        self,
+        task_name: str,
+        delete_output: bool = False,
+    ) -> bool:
+        """Delete a task and prevent any further runs.
 
-    collection_name.add(ids=[...], documents=[...]) # Raises the error defined below
-    # __getattribute__ is invoked first, not finding a match in str.
-    # __getattr__ from this class is invoked and raises an error
+        Args:
+            task_name: Name of the task to remove
+            delete_output: Whether to also delete the output collection. Defaults to False.
 
-    """
+        Returns:
+            bool: True if successful
 
-    def __getattr__(self, item):
-        collection_attributes_and_methods = [
-            member
-            for member, _ in inspect.getmembers(Collection)
-            if not member.startswith("_")
-        ]
-
-        if item in collection_attributes_and_methods:
-            raise NotImplementedError(
-                f"In Chroma v0.6.0, list_collections only returns collection names. "
-                f"Use Client.get_collection({str(self)}) to access {item}. "
-                f"See https://docs.trychroma.com/deployment/migration for more information."
-            )
-
-        raise AttributeError(f"'CollectionName' object has no attribute '{item}'")
+        Example:
+            >>> success = collection.remove_task("count_docs", delete_output=True)
+        """
+        return self._client.remove_task(
+            task_name=task_name,
+            input_collection_id=self.id,
+            delete_output=delete_output,
+            tenant=self.tenant,
+            database=self.database,
+        )
