@@ -1,7 +1,14 @@
 from typing import List, Optional, Sequence, Tuple, Union, cast
 from uuid import UUID
 from overrides import overrides
-from chromadb.api.configuration import CollectionConfigurationInternal
+from chromadb.api.collection_configuration import (
+    CreateCollectionConfiguration,
+    create_collection_configuration_to_json_str,
+    UpdateCollectionConfiguration,
+    update_collection_configuration_to_json_str,
+    CollectionMetadata,
+)
+from chromadb.api.types import Schema
 from chromadb.config import DEFAULT_DATABASE, DEFAULT_TENANT, System, logger
 from chromadb.db.system import SysDB
 from chromadb.errors import NotFoundError, UniqueConstraintError, InternalError
@@ -17,11 +24,15 @@ from chromadb.proto.coordinator_pb2 import (
     CreateDatabaseRequest,
     CreateSegmentRequest,
     CreateTenantRequest,
+    CountCollectionsRequest,
+    CountCollectionsResponse,
     DeleteCollectionRequest,
     DeleteDatabaseRequest,
     DeleteSegmentRequest,
     GetCollectionsRequest,
     GetCollectionsResponse,
+    GetCollectionSizeRequest,
+    GetCollectionSizeResponse,
     GetCollectionWithSegmentsRequest,
     GetCollectionWithSegmentsResponse,
     GetDatabaseRequest,
@@ -34,6 +45,10 @@ from chromadb.proto.coordinator_pb2 import (
 from chromadb.proto.coordinator_pb2_grpc import SysDBStub
 from chromadb.proto.utils import RetryOnRpcErrorClientInterceptor
 from chromadb.telemetry.opentelemetry.grpc import OtelInterceptor
+from chromadb.telemetry.opentelemetry import (
+    OpenTelemetryGranularity,
+    trace_method,
+)
 from chromadb.types import (
     Collection,
     CollectionAndSegments,
@@ -74,6 +89,7 @@ class GrpcSysDB(SysDB):
     def start(self) -> None:
         self._channel = grpc.insecure_channel(
             f"{self._coordinator_url}:{self._coordinator_port}",
+            options=[("grpc.max_concurrent_streams", 1000)],
         )
         interceptors = [OtelInterceptor(), RetryOnRpcErrorClientInterceptor()]
         self._channel = grpc.intercept_channel(self._channel, *interceptors)
@@ -300,7 +316,8 @@ class GrpcSysDB(SysDB):
         self,
         id: UUID,
         name: str,
-        configuration: CollectionConfigurationInternal,
+        schema: Optional[Schema],
+        configuration: CreateCollectionConfiguration,
         segments: Sequence[Segment],
         metadata: Optional[Metadata] = None,
         dimension: Optional[int] = None,
@@ -312,7 +329,9 @@ class GrpcSysDB(SysDB):
             request = CreateCollectionRequest(
                 id=id.hex,
                 name=name,
-                configuration_json_str=configuration.to_json_str(),
+                configuration_json_str=create_collection_configuration_to_json_str(
+                    configuration, cast(CollectionMetadata, metadata)
+                ),
                 metadata=to_proto_update_metadata(metadata) if metadata else None,
                 dimension=dimension,
                 get_or_create=get_or_create,
@@ -412,6 +431,47 @@ class GrpcSysDB(SysDB):
             )
             raise InternalError()
 
+    @overrides
+    def count_collections(
+        self,
+        tenant: str = DEFAULT_TENANT,
+        database: Optional[str] = None,
+    ) -> int:
+        try:
+            if database is None or database == "":
+                request = CountCollectionsRequest(tenant=tenant)
+                response: CountCollectionsResponse = self._sys_db_stub.CountCollections(
+                    request
+                )
+                return response.count
+            else:
+                request = CountCollectionsRequest(
+                    tenant=tenant,
+                    database=database,
+                )
+                response: CountCollectionsResponse = self._sys_db_stub.CountCollections(
+                    request
+                )
+                return response.count
+        except grpc.RpcError as e:
+            logger.error(f"Failed to count collections due to error: {e}")
+            raise InternalError()
+
+    @overrides
+    def get_collection_size(self, id: UUID) -> int:
+        try:
+            request = GetCollectionSizeRequest(id=id.hex)
+            response: GetCollectionSizeResponse = self._sys_db_stub.GetCollectionSize(
+                request
+            )
+            return response.total_records_post_compaction
+        except grpc.RpcError as e:
+            logger.error(f"Failed to get collection {id} size due to error: {e}")
+            raise InternalError()
+
+    @trace_method(
+        "SysDB.get_collection_with_segments", OpenTelemetryGranularity.OPERATION
+    )
     @overrides
     def get_collection_with_segments(
         self, collection_id: UUID
@@ -440,6 +500,9 @@ class GrpcSysDB(SysDB):
         name: OptionalArgument[str] = Unspecified(),
         dimension: OptionalArgument[Optional[int]] = Unspecified(),
         metadata: OptionalArgument[Optional[UpdateMetadata]] = Unspecified(),
+        configuration: OptionalArgument[
+            Optional[UpdateCollectionConfiguration]
+        ] = Unspecified(),
     ) -> None:
         try:
             write_name = None
@@ -454,6 +517,12 @@ class GrpcSysDB(SysDB):
             if metadata != Unspecified():
                 write_metadata = cast(Union[UpdateMetadata, None], metadata)
 
+            write_configuration = None
+            if configuration != Unspecified():
+                write_configuration = cast(
+                    Union[UpdateCollectionConfiguration, None], configuration
+                )
+
             request = UpdateCollectionRequest(
                 id=id.hex,
                 name=write_name,
@@ -461,6 +530,11 @@ class GrpcSysDB(SysDB):
                 metadata=to_proto_update_metadata(write_metadata)
                 if write_metadata
                 else None,
+                configuration_json_str=update_collection_configuration_to_json_str(
+                    write_configuration
+                )
+                if write_configuration
+                else None,
             )
             if metadata is None:
                 request.ClearField("metadata")
