@@ -137,12 +137,14 @@ impl UnorderedBlockDelta {
 mod test {
     use crate::arrow::{
         block::{delta::UnorderedBlockDelta, Block},
-        config::TEST_MAX_BLOCK_SIZE_BYTES,
+        config::{BlockManagerConfig, TEST_MAX_BLOCK_SIZE_BYTES},
         provider::BlockManager,
     };
     #[cfg(test)]
     use chroma_cache::new_cache_for_test;
-    use chroma_storage::{local::LocalStorage, Storage};
+    use chroma_storage::{
+        admissioncontrolleds3::StorageRequestPriority, local::LocalStorage, Storage,
+    };
     use chroma_types::{DataRecord, MetadataValue};
     use rand::{random, Rng};
     use roaring::RoaringBitmap;
@@ -169,7 +171,12 @@ mod test {
         let path = tmp_dir.path().to_str().unwrap();
         let storage = Storage::Local(LocalStorage::new(path));
         let cache = new_cache_for_test();
-        let block_manager = BlockManager::new(storage, TEST_MAX_BLOCK_SIZE_BYTES, cache);
+        let block_manager = BlockManager::new(
+            storage,
+            TEST_MAX_BLOCK_SIZE_BYTES,
+            cache,
+            BlockManagerConfig::default_num_concurrent_block_flushes(),
+        );
         let delta = block_manager.create::<&str, Vec<u32>, UnorderedBlockDelta>();
 
         let n = 2000;
@@ -193,8 +200,13 @@ mod test {
             let read = block.get::<&str, &[u32]>("prefix", &key).unwrap();
             values_before_flush.push(read.to_vec());
         }
-        block_manager.flush(&block).await.unwrap();
-        let block = block_manager.get(&block.clone().id).await.unwrap().unwrap();
+        let prefix_path = "";
+        block_manager.flush(&block, prefix_path).await.unwrap();
+        let block = block_manager
+            .get(prefix_path, &block.id, StorageRequestPriority::P0)
+            .await
+            .unwrap()
+            .unwrap();
         #[allow(clippy::needless_range_loop)]
         for i in 0..n {
             let key = format!("key{}", i);
@@ -211,7 +223,12 @@ mod test {
         let path = tmp_dir.path().to_str().unwrap();
         let storage = Storage::Local(LocalStorage::new(tmp_dir.path().to_str().unwrap()));
         let cache = new_cache_for_test();
-        let block_manager = BlockManager::new(storage, TEST_MAX_BLOCK_SIZE_BYTES, cache);
+        let block_manager = BlockManager::new(
+            storage,
+            TEST_MAX_BLOCK_SIZE_BYTES,
+            cache,
+            BlockManagerConfig::default_num_concurrent_block_flushes(),
+        );
         let delta = block_manager.create::<&str, String, UnorderedBlockDelta>();
         let delta_id = delta.id;
 
@@ -232,9 +249,14 @@ mod test {
             let read = block.get::<&str, &str>("prefix", &key);
             values_before_flush.push(read.unwrap().to_string());
         }
-        block_manager.flush(&block).await.unwrap();
+        let prefix_path = "";
+        block_manager.flush(&block, prefix_path).await.unwrap();
 
-        let block = block_manager.get(&delta_id).await.unwrap().unwrap();
+        let block = block_manager
+            .get(prefix_path, &delta_id, StorageRequestPriority::P0)
+            .await
+            .unwrap()
+            .unwrap();
 
         assert_eq!(size, block.get_size());
         #[allow(clippy::needless_range_loop)]
@@ -255,13 +277,17 @@ mod test {
 
         // test fork
         let forked_block = block_manager
-            .fork::<&str, String, UnorderedBlockDelta>(&delta_id)
+            .fork::<&str, String, UnorderedBlockDelta>(&delta_id, prefix_path)
             .await
             .unwrap();
         let new_id = forked_block.id;
         let block = block_manager.commit::<&str, String>(forked_block).await;
-        block_manager.flush(&block).await.unwrap();
-        let forked_block = block_manager.get(&new_id).await.unwrap().unwrap();
+        block_manager.flush(&block, prefix_path).await.unwrap();
+        let forked_block = block_manager
+            .get(prefix_path, &new_id, StorageRequestPriority::P0)
+            .await
+            .unwrap()
+            .unwrap();
         for i in 0..n {
             let key = format!("key{}", i);
             let read = forked_block.get::<&str, &str>("prefix", &key);
@@ -275,7 +301,12 @@ mod test {
         let path = tmp_dir.path().to_str().unwrap();
         let storage = Storage::Local(LocalStorage::new(path));
         let cache = new_cache_for_test();
-        let block_manager = BlockManager::new(storage, TEST_MAX_BLOCK_SIZE_BYTES, cache);
+        let block_manager = BlockManager::new(
+            storage,
+            TEST_MAX_BLOCK_SIZE_BYTES,
+            cache,
+            BlockManagerConfig::default_num_concurrent_block_flushes(),
+        );
         let delta = block_manager.create::<f32, String, UnorderedBlockDelta>();
 
         let n = 2000;
@@ -295,8 +326,13 @@ mod test {
             let read = block.get::<f32, &str>("prefix", key).unwrap();
             values_before_flush.push(read);
         }
-        block_manager.flush(&block).await.unwrap();
-        let block = block_manager.get(&delta_id).await.unwrap().unwrap();
+        let prefix_path = "";
+        block_manager.flush(&block, prefix_path).await.unwrap();
+        let block = block_manager
+            .get(prefix_path, &delta_id, StorageRequestPriority::P0)
+            .await
+            .unwrap()
+            .unwrap();
         assert_eq!(size, block.get_size());
         #[allow(clippy::needless_range_loop)]
         for i in 0..n {
@@ -314,7 +350,12 @@ mod test {
         let path = tmp_dir.path().to_str().unwrap();
         let storage = Storage::Local(LocalStorage::new(path));
         let cache = new_cache_for_test();
-        let block_manager = BlockManager::new(storage, TEST_MAX_BLOCK_SIZE_BYTES, cache);
+        let block_manager = BlockManager::new(
+            storage,
+            TEST_MAX_BLOCK_SIZE_BYTES,
+            cache,
+            BlockManagerConfig::default_num_concurrent_block_flushes(),
+        );
         let delta = block_manager.create::<&str, RoaringBitmap, UnorderedBlockDelta>();
 
         let n = 2000;
@@ -328,8 +369,13 @@ mod test {
         let size = delta.get_size::<&str, RoaringBitmap>();
         let delta_id = delta.id;
         let block = block_manager.commit::<&str, RoaringBitmap>(delta).await;
-        block_manager.flush(&block).await.unwrap();
-        let block = block_manager.get(&delta_id).await.unwrap().unwrap();
+        let prefix_path = "";
+        block_manager.flush(&block, prefix_path).await.unwrap();
+        let block = block_manager
+            .get(prefix_path, &delta_id, StorageRequestPriority::P0)
+            .await
+            .unwrap()
+            .unwrap();
 
         assert_eq!(size, block.get_size());
 
@@ -350,7 +396,12 @@ mod test {
         let path = tmp_dir.path().to_str().unwrap();
         let storage = Storage::Local(LocalStorage::new(path));
         let cache = new_cache_for_test();
-        let block_manager = BlockManager::new(storage, TEST_MAX_BLOCK_SIZE_BYTES, cache);
+        let block_manager = BlockManager::new(
+            storage,
+            TEST_MAX_BLOCK_SIZE_BYTES,
+            cache,
+            BlockManagerConfig::default_num_concurrent_block_flushes(),
+        );
         let ids = ["embedding_id_2", "embedding_id_0", "embedding_id_1"];
         let embeddings = [
             vec![1.0, 2.0, 3.0],
@@ -393,8 +444,13 @@ mod test {
         let size = delta.get_size::<&str, &DataRecord>();
         let delta_id = delta.id;
         let block = block_manager.commit::<&str, &DataRecord>(delta).await;
-        block_manager.flush(&block).await.unwrap();
-        let block = block_manager.get(&delta_id).await.unwrap().unwrap();
+        let prefix_path = "";
+        block_manager.flush(&block, prefix_path).await.unwrap();
+        let block = block_manager
+            .get(prefix_path, &delta_id, StorageRequestPriority::P0)
+            .await
+            .unwrap()
+            .unwrap();
         for i in 0..3 {
             let read = block.get::<&str, DataRecord>("", ids[i]).unwrap();
             assert_eq!(read.id, ids[i]);
@@ -414,7 +470,12 @@ mod test {
         let path = tmp_dir.path().to_str().unwrap();
         let storage = Storage::Local(LocalStorage::new(path));
         let cache = new_cache_for_test();
-        let block_manager = BlockManager::new(storage, TEST_MAX_BLOCK_SIZE_BYTES, cache);
+        let block_manager = BlockManager::new(
+            storage,
+            TEST_MAX_BLOCK_SIZE_BYTES,
+            cache,
+            BlockManagerConfig::default_num_concurrent_block_flushes(),
+        );
         let delta = block_manager.create::<u32, String, UnorderedBlockDelta>();
 
         let n = 2000;
@@ -428,8 +489,13 @@ mod test {
         let size = delta.get_size::<u32, String>();
         let delta_id = delta.id;
         let block = block_manager.commit::<u32, String>(delta).await;
-        block_manager.flush(&block).await.unwrap();
-        let block = block_manager.get(&delta_id).await.unwrap().unwrap();
+        let prefix_path = "";
+        block_manager.flush(&block, prefix_path).await.unwrap();
+        let block = block_manager
+            .get(prefix_path, &delta_id, StorageRequestPriority::P0)
+            .await
+            .unwrap()
+            .unwrap();
         assert_eq!(size, block.get_size());
 
         // test save/load
@@ -442,7 +508,12 @@ mod test {
         let path = tmp_dir.path().to_str().unwrap();
         let storage = Storage::Local(LocalStorage::new(tmp_dir.path().to_str().unwrap()));
         let cache = new_cache_for_test();
-        let block_manager = BlockManager::new(storage, TEST_MAX_BLOCK_SIZE_BYTES, cache);
+        let block_manager = BlockManager::new(
+            storage,
+            TEST_MAX_BLOCK_SIZE_BYTES,
+            cache,
+            BlockManagerConfig::default_num_concurrent_block_flushes(),
+        );
         let delta = block_manager.create::<u32, u32, UnorderedBlockDelta>();
         let delta_id = delta.id;
 
@@ -463,9 +534,14 @@ mod test {
             let read = block.get::<u32, u32>("prefix", key);
             values_before_flush.push(read.unwrap().to_string());
         }
-        block_manager.flush(&block).await.unwrap();
+        let prefix_path = "";
+        block_manager.flush(&block, prefix_path).await.unwrap();
 
-        let block = block_manager.get(&delta_id).await.unwrap().unwrap();
+        let block = block_manager
+            .get(prefix_path, &delta_id, StorageRequestPriority::P0)
+            .await
+            .unwrap()
+            .unwrap();
 
         assert_eq!(size, block.get_size());
         #[allow(clippy::needless_range_loop)]
@@ -486,13 +562,17 @@ mod test {
 
         // test fork
         let forked_block = block_manager
-            .fork::<u32, u32, UnorderedBlockDelta>(&delta_id)
+            .fork::<u32, u32, UnorderedBlockDelta>(&delta_id, prefix_path)
             .await
             .unwrap();
         let new_id = forked_block.id;
         let block = block_manager.commit::<u32, u32>(forked_block).await;
-        block_manager.flush(&block).await.unwrap();
-        let forked_block = block_manager.get(&new_id).await.unwrap().unwrap();
+        block_manager.flush(&block, prefix_path).await.unwrap();
+        let forked_block = block_manager
+            .get(prefix_path, &new_id, StorageRequestPriority::P0)
+            .await
+            .unwrap()
+            .unwrap();
         #[allow(clippy::needless_range_loop)]
         for i in 0..n {
             let key = i as u32;
