@@ -3,7 +3,7 @@ use crate::arrow::{
     types::{ArrowReadableKey, ArrowReadableValue, ArrowWriteableKey},
 };
 use arrow::{
-    array::{Array, StringArray, StringBuilder},
+    array::{Array, AsArray, StringArray, StringBuilder},
     util::bit_util,
 };
 use std::sync::Arc;
@@ -33,6 +33,14 @@ impl<'referred_data> ArrowReadableKey<'referred_data> for &'referred_data str {
             .unwrap()
             .value(index)
     }
+
+    fn get_range(array: &'referred_data Arc<dyn Array>, offset: usize, length: usize) -> Vec<Self> {
+        let str_array = array.as_string::<i32>();
+        (offset..offset + length)
+            .map(|i| str_array.value(i))
+            .collect()
+    }
+
     fn add_to_delta<'external, V: ArrowReadableValue<'external>>(
         prefix: &str,
         key: Self,
