@@ -2,7 +2,10 @@ use crate::arrow::{
     block::delta::{BlockKeyArrowBuilder, BlockStorage},
     types::{ArrowReadableKey, ArrowReadableValue, ArrowWriteableKey},
 };
-use arrow::array::{Array, StringBuilder, UInt32Array, UInt32Builder};
+use arrow::{
+    array::{Array, AsArray, StringBuilder, UInt32Array, UInt32Builder},
+    datatypes::UInt32Type,
+};
 use std::sync::Arc;
 
 impl ArrowWriteableKey for u32 {
@@ -31,6 +34,14 @@ impl ArrowReadableKey<'_> for u32 {
             .value(index)
     }
 
+    fn get_range(array: &Arc<dyn Array>, offset: usize, length: usize) -> Vec<Self> {
+        array
+            .as_primitive::<UInt32Type>()
+            .slice(offset, length)
+            .values()
+            .to_vec()
+    }
+
     fn add_to_delta<'external, V: ArrowReadableValue<'external>>(
         prefix: &str,
         key: Self,
