@@ -1,18 +1,21 @@
 use std::cmp::Ordering;
 use std::collections::HashMap;
-use std::io::SeekFrom;
+use std::io::Read;
 use std::ops::{Bound, RangeBounds};
+use std::sync::Arc;
 
 use crate::arrow::types::{ArrowReadableKey, ArrowReadableValue};
 use arrow::array::ArrayData;
 use arrow::buffer::Buffer;
-use arrow::ipc::reader::read_footer_length;
-use arrow::ipc::{root_as_footer, root_as_message, MessageHeader, MetadataVersion};
+use arrow::ipc::convert::fb_to_schema;
+use arrow::ipc::reader::{read_footer_length, FileDecoder};
+use arrow::ipc::{root_as_footer, root_as_message, Footer, MessageHeader, MetadataVersion};
 use arrow::util::bit_util;
 use arrow::{
     array::{Array, StringArray},
     record_batch::RecordBatch,
 };
+use chroma_cache::foyer::MIB;
 use chroma_error::{ChromaError, ErrorCodes};
 use serde::de::Error as DeError;
 use serde::ser::Error as SerError;
@@ -66,8 +69,7 @@ impl<'de> Deserialize<'de> for RecordBatchWrapper {
         D: serde::Deserializer<'de>,
     {
         let data = Vec::<u8>::deserialize(deserializer)?;
-        let reader = std::io::Cursor::new(data);
-        let rb = Block::load_record_batch(reader, false).map_err(D::Error::custom)?;
+        let rb = Block::load_record_batch(&data, false).map_err(D::Error::custom)?;
         Ok(RecordBatchWrapper(rb))
     }
 }
@@ -203,75 +205,33 @@ impl Block {
         }
     }
 
-    /// Returns the largest index where `prefixes[index] == prefix` or None if the provided prefix does not exist in the block.
-    #[inline]
-    fn find_largest_index_of_prefix<'me, K: ArrowReadableKey<'me>>(
-        &'me self,
-        prefix: &str,
-    ) -> Option<usize> {
-        // By design, will never find an exact match (comparator never evaluates to Equal). This finds the index of the first element that is greater than the prefix. If no element is greater, it returns the length of the array.
-        let result = self
-            .binary_search_by::<K, _>(|(p, _)| match p.cmp(prefix) {
-                Ordering::Less => Ordering::Less,
-                Ordering::Equal => Ordering::Less,
-                Ordering::Greater => Ordering::Greater,
-            })
-            .expect_err("Never returns Ok because the comparator never evaluates to Equal.");
-
-        if result == 0 {
-            // The first element is greater than the target prefix, so the target prefix does not exist in the block. (Or the block is empty.)
-            return None;
-        }
-
-        let prefix_array = self
-            .data
-            .column(0)
-            .as_any()
-            .downcast_ref::<StringArray>()
-            .unwrap();
-
-        // `result` is the first index where the prefix is larger than the input (or the length of the array) so we want one element before this.
-        match prefix_array.value(result - 1).cmp(prefix) {
-            // We're at the end of the array, so the prefix does not exist in the block (all values are less than the prefix)
-            Ordering::Less => None,
-            // The prefix exists
-            Ordering::Equal => Some(result - 1),
-            // This is impossible
-            Ordering::Greater => None,
-        }
-    }
-
-    /// Returns the smallest index where `prefixes[index] == prefix` or None if the provided prefix does not exist in the block.
+    /// Returns the smallest index where `prefixes[index] >= prefix`. If such index does not exist `prefixes.len()` will be returned.
     #[inline]
     fn find_smallest_index_of_prefix<'me, K: ArrowReadableKey<'me>>(
         &'me self,
         prefix: &str,
-    ) -> Option<usize> {
-        let result = self
-            .binary_search_by::<K, _>(|(p, _)| match p.cmp(prefix) {
-                Ordering::Less => Ordering::Less,
-                Ordering::Equal => Ordering::Greater,
-                Ordering::Greater => Ordering::Greater,
-            })
-            .expect_err("Never returns Ok because the comparator never evaluates to Equal.");
+    ) -> usize {
+        self.binary_search_by::<K, _>(|(p, _)| match p.cmp(prefix) {
+            Ordering::Less => Ordering::Less,
+            Ordering::Equal => Ordering::Greater,
+            Ordering::Greater => Ordering::Greater,
+        })
+        .expect_err("Never returns Ok because the comparator never evaluates to Equal.")
+    }
 
-        let prefix_array = self
-            .data
-            .column(0)
-            .as_any()
-            .downcast_ref::<StringArray>()
-            .unwrap();
-
-        if result == self.len() {
-            // The target prefix is greater than all elements in the block.
-            return None;
-        }
-
-        match prefix_array.value(result).cmp(prefix) {
-            Ordering::Greater => None,
-            Ordering::Less => None,
-            Ordering::Equal => Some(result),
-        }
+    /// Returns the smallest index where `prefixes[index] > prefix`. If such index does not exist `prefixes.len()` will be returned.
+    #[inline]
+    fn find_smallest_index_of_next_prefix<'me, K: ArrowReadableKey<'me>>(
+        &'me self,
+        prefix: &str,
+    ) -> usize {
+        // By design, will never find an exact match (comparator never evaluates to Equal). This finds the index of the first element that is greater than the prefix. If no element is greater, it returns the length of the array.
+        self.binary_search_by::<K, _>(|(p, _)| match p.cmp(prefix) {
+            Ordering::Less => Ordering::Less,
+            Ordering::Equal => Ordering::Less,
+            Ordering::Greater => Ordering::Greater,
+        })
+        .expect_err("Never returns Ok because the comparator never evaluates to Equal.")
     }
 
     /// Finds the partition point of the prefix and key.
@@ -325,6 +285,30 @@ impl Block {
         ===== Block Queries =====
     */
 
+    /// Get all key-value pairs for a specific prefix in the block
+    /// Returns an iterator of (key, value) pairs for better performance when collecting all values
+    pub fn get_prefix<'me, K: ArrowReadableKey<'me>, V: ArrowReadableValue<'me>>(
+        &'me self,
+        prefix: &str,
+    ) -> impl Iterator<Item = (K, V)> {
+        // Find the start index for this prefix
+        let offset = self.find_smallest_index_of_prefix::<K>(prefix);
+        if offset >= self.len() {
+            return Vec::new().into_iter().zip(Vec::new());
+        }
+
+        // Find the end index (first element with a different prefix)
+        let cap = self.find_smallest_index_of_next_prefix::<K>(prefix);
+        let length = cap - offset;
+
+        // Extract key value pairs
+        let keys = K::get_range(self.data.column(1), offset, length);
+        let values = V::get_range(self.data.column(2), offset, length);
+
+        // Zip and collect
+        keys.into_iter().zip(values)
+    }
+
     /// Get the value for a given key in the block
     /// ### Panics
     /// - If the underlying data types are not the same as the types specified in the function signature
@@ -347,9 +331,12 @@ impl Block {
     }
 
     /// Get all the values for a given prefix & key range in the block
-    /// ### Panics
-    /// - If the underlying data types are not the same as the types specified in the function signature
-    /// - If at least one end of the prefix range is excluded (currently unsupported)
+    ///
+    /// The prefix and key of the returning value must be contained by the prefix range and key range respectively
+    ///
+    /// ### Example
+    /// If we have block: [(p0, k0, v0), (p0, k1, v1), (p1, k0, v2), (p1, k1, v3), (p2, k1, v4)]
+    /// Then block.get_range(p0..p2, k1..) will return [(p0, k1, v1), (p1, k1, v3)]
     pub fn get_range<
         'prefix,
         'me,
@@ -361,61 +348,92 @@ impl Block {
         &'me self,
         prefix_range: PrefixRange,
         key_range: KeyRange,
-    ) -> impl Iterator<Item = (K, V)> + 'me
+    ) -> impl Iterator<Item = (&'me str, K, V)> + 'me
     where
         PrefixRange: RangeBounds<&'prefix str>,
         KeyRange: RangeBounds<K>,
     {
-        let start_index = match prefix_range.start_bound() {
-            Bound::Included(prefix) => match key_range.start_bound() {
-                Bound::Included(key) => self.binary_search_prefix_key(prefix, key),
-                Bound::Excluded(key) => {
-                    let index = self.binary_search_prefix_key(prefix, key);
-                    if self.match_prefix_key_at_index(prefix, key, index) {
-                        index + 1
-                    } else {
-                        index
-                    }
-                }
-                Bound::Unbounded => match self.find_smallest_index_of_prefix::<K>(prefix) {
-                    Some(first_index_of_prefix) => first_index_of_prefix,
-                    None => self.len(), // prefix does not exist in the block so we shouldn't return anything
-                },
-            },
-            Bound::Excluded(_) => {
-                unimplemented!("Excluded prefix range is not currently supported")
-            }
-            Bound::Unbounded => 0,
-        };
+        let mut index_ranges = Vec::new();
 
-        let end_index = match prefix_range.end_bound() {
-            Bound::Included(prefix) => match key_range.end_bound() {
-                Bound::Included(key) => {
-                    let index = self.binary_search_prefix_key(prefix, key);
-                    if self.match_prefix_key_at_index(prefix, key, index) {
-                        index + 1
-                    } else {
-                        index
-                    }
-                }
-                Bound::Excluded(key) => self.binary_search_prefix_key::<K>(prefix, key),
-                Bound::Unbounded => match self.find_largest_index_of_prefix::<K>(prefix) {
-                    Some(last_index_of_prefix) => last_index_of_prefix + 1, // (add 1 because end_index is exclusive below)
-                    None => start_index, // prefix does not exist in the block so we shouldn't return anything
-                },
-            },
-            Bound::Excluded(_) => {
-                unimplemented!("Excluded prefix range is not currently supported")
-            }
-            Bound::Unbounded => self.len(),
-        };
+        let prefix_array = self
+            .data
+            .column(0)
+            .as_any()
+            .downcast_ref::<StringArray>()
+            .unwrap();
 
-        (start_index..end_index).map(move |index| {
+        let prefix_key_val_getter = |index| {
             (
+                prefix_array.value(index),
                 K::get(self.data.column(1), index),
                 V::get(self.data.column(2), index),
             )
-        })
+        };
+
+        let mut cursor_prefix_index = match prefix_range.start_bound() {
+            Bound::Included(prefix) => self.find_smallest_index_of_prefix::<K>(prefix),
+            Bound::Excluded(prefix) => self.find_smallest_index_of_next_prefix::<K>(prefix),
+            Bound::Unbounded => 0,
+        };
+
+        if let (Bound::Unbounded, Bound::Unbounded) =
+            (key_range.start_bound(), key_range.end_bound())
+        {
+            let final_prefix_index = match prefix_range.end_bound() {
+                Bound::Included(prefix) => self.find_smallest_index_of_next_prefix::<K>(prefix),
+                Bound::Excluded(prefix) => self.find_smallest_index_of_prefix::<K>(prefix),
+                Bound::Unbounded => self.len(),
+            };
+            index_ranges.push(cursor_prefix_index..final_prefix_index);
+            return index_ranges
+                .into_iter()
+                .flatten()
+                .map(prefix_key_val_getter);
+        }
+
+        while cursor_prefix_index < self.len() {
+            let cursor_prefix = prefix_array.value(cursor_prefix_index);
+            if !prefix_range.contains(&cursor_prefix) {
+                break;
+            }
+            let next_cursor_prefix_index =
+                self.find_smallest_index_of_next_prefix::<K>(cursor_prefix);
+
+            let cursor_prefix_range_start_index = match key_range.start_bound() {
+                Bound::Included(start_key) => {
+                    self.binary_search_prefix_key(cursor_prefix, start_key)
+                }
+                Bound::Excluded(start_key) => {
+                    let index = self.binary_search_prefix_key(cursor_prefix, start_key);
+                    index + self.match_prefix_key_at_index(cursor_prefix, start_key, index) as usize
+                }
+                Bound::Unbounded => cursor_prefix_index,
+            };
+
+            let cursor_prefix_range_end_index = match key_range.end_bound() {
+                Bound::Included(end_key) => {
+                    let index = self.binary_search_prefix_key(cursor_prefix, end_key);
+                    index + self.match_prefix_key_at_index(cursor_prefix, end_key, index) as usize
+                }
+                Bound::Excluded(end_key) => {
+                    self.binary_search_prefix_key::<K>(cursor_prefix, end_key)
+                }
+                Bound::Unbounded => next_cursor_prefix_index,
+            };
+
+            let cursor_prefix_range =
+                cursor_prefix_range_start_index..cursor_prefix_range_end_index;
+            if !cursor_prefix_range.is_empty() {
+                index_ranges.push(cursor_prefix_range);
+            }
+
+            cursor_prefix_index = next_cursor_prefix_index;
+        }
+
+        index_ranges
+            .into_iter()
+            .flatten()
+            .map(prefix_key_val_getter)
     }
 
     /*
@@ -528,7 +546,7 @@ impl Block {
 
     /// Load a block from bytes in Arrow IPC format with the given id
     pub fn from_bytes(bytes: &[u8], id: Uuid) -> Result<Self, BlockLoadError> {
-        Self::from_bytes_internal(bytes, id, false)
+        Self::load_from_bytes(bytes, id, false)
     }
 
     /// Load a block from bytes in Arrow IPC format with the given id and validate the layout
@@ -536,12 +554,7 @@ impl Block {
     /// - This method should be used in tests to ensure that the layout of the IPC file is as expected
     /// - The validation is not performant and should not be used in production code
     pub fn from_bytes_with_validation(bytes: &[u8], id: Uuid) -> Result<Self, BlockLoadError> {
-        Self::from_bytes_internal(bytes, id, true)
-    }
-
-    fn from_bytes_internal(bytes: &[u8], id: Uuid, validate: bool) -> Result<Self, BlockLoadError> {
-        let cursor = std::io::Cursor::new(bytes);
-        Self::load_with_reader(cursor, id, validate)
+        Self::load_from_bytes(bytes, id, true)
     }
 
     /// Load a block from the given path with the given id and validate the layout
@@ -565,47 +578,53 @@ impl Block {
                 return Err(BlockLoadError::IOError(e));
             }
         };
-        let reader = std::io::BufReader::new(file);
-        Self::load_with_reader(reader, id, validate)
+        let mut reader = std::io::BufReader::new(file);
+        let mut target_buffer = Vec::with_capacity(reader.get_ref().metadata()?.len() as usize);
+        reader.read_to_end(&mut target_buffer)?;
+        Self::load_from_bytes(&target_buffer, id, validate)
     }
 
-    fn load_with_reader<R>(reader: R, id: Uuid, validate: bool) -> Result<Self, BlockLoadError>
-    where
-        R: std::io::Read + std::io::Seek,
-    {
-        let batch = Self::load_record_batch(reader, validate)?;
-        // TODO: how to store / hydrate id?
+    fn load_from_bytes(bytes: &[u8], id: Uuid, validate: bool) -> Result<Self, BlockLoadError> {
+        let batch = Self::load_record_batch(bytes, validate)?;
         Ok(Self::from_record_batch(id, batch))
     }
 
-    fn load_record_batch<R>(mut reader: R, validate: bool) -> Result<RecordBatch, BlockLoadError>
-    where
-        R: std::io::Read + std::io::Seek,
-    {
+    fn load_record_batch(bytes: &[u8], validate: bool) -> Result<RecordBatch, BlockLoadError> {
         if validate {
-            verify_buffers_layout(&mut reader)
-                .map_err(BlockLoadError::ArrowLayoutVerificationError)?;
+            verify_buffers_layout(bytes).map_err(BlockLoadError::ArrowLayoutVerificationError)?;
         }
 
-        let mut arrow_reader = arrow::ipc::reader::FileReader::try_new(&mut reader, None)
-            .map_err(BlockLoadError::ArrowError)?;
+        let footer =
+            read_arrow_footer(bytes).map_err(BlockLoadError::ArrowLayoutVerificationError)?;
+        let schema = footer
+            .schema()
+            .ok_or(BlockLoadError::ArrowLayoutVerificationError(
+                ArrowLayoutVerificationError::RecordBatchDecodeError,
+            ))?;
+        let schema = fb_to_schema(schema);
+        // Requiring alignment should always work for blocks since we write them with alignment
+        // This is just being defensive
+        let decoder =
+            FileDecoder::new(Arc::new(schema), footer.version()).with_require_alignment(true);
+        let (block, record_batch_offset, _, record_batch_len) = read_record_batch_range(footer)?;
 
-        let batch = match arrow_reader.next() {
-            Some(Ok(batch)) => batch,
-            Some(Err(e)) => {
-                return Err(BlockLoadError::ArrowError(e));
-            }
-            None => {
-                return Err(BlockLoadError::NoRecordBatches);
-            }
-        };
-        Ok(batch)
+        // This incurs a copy of the buffer we should be able to avoid this
+        // but as is foyer hands a reference to the [u8]. So the end to end
+        // path involves up to two copies, kernel to user space copy when reading from disk cache
+        // and then a copy into this buffer. We could avoid the second copy by changing foyer to
+        // hand over ownership of the buffer, but that would be a larger change.
+        // This is something we can optimize later if it becomes a bottleneck.
+        let buffer =
+            Buffer::from(&bytes[record_batch_offset..record_batch_offset + record_batch_len]);
+        decoder
+            .read_record_batch(block, &buffer)?
+            .ok_or(BlockLoadError::NoRecordBatches)
     }
 }
 
 impl chroma_cache::Weighted for Block {
     fn weight(&self) -> usize {
-        8 // A block is at most 8 MB
+        ((self.get_size() as f32 / MIB as f32).ceil() as usize).max(1)
     }
 }
 
@@ -744,40 +763,25 @@ impl ChromaError for ArrowLayoutVerificationError {
     }
 }
 
-/// Verifies that the buffers in the IPC file are 64 byte aligned
-/// and stored in Arrow in the way we expect.
-/// All non-benchmark test code should use this by loading the block
-/// with verification enabled.
-fn verify_buffers_layout<R>(mut reader: R) -> Result<(), ArrowLayoutVerificationError>
-where
-    R: std::io::Read + std::io::Seek,
-{
-    // Read the IPC file and verify that the buffers are 64 byte aligned
-    // by inspecting the offsets, this is required since our
-    // size calculation assumes that the buffers are 64 byte aligned
+fn read_arrow_footer(bytes: &[u8]) -> Result<Footer<'_>, ArrowLayoutVerificationError> {
     // Space for ARROW_MAGIC (6 bytes) and length (4 bytes)
     let mut footer_buffer = [0; 10];
-    reader
-        .seek(SeekFrom::End(-10))
-        .map_err(ArrowLayoutVerificationError::IOError)?;
-    reader
-        .read_exact(&mut footer_buffer)
-        .map_err(ArrowLayoutVerificationError::IOError)?;
-
-    let footer_len = read_footer_length(footer_buffer);
-    let footer_len = footer_len.map_err(ArrowLayoutVerificationError::ArrowError)?;
+    let trailer_start = bytes.len() - 10;
+    footer_buffer.copy_from_slice(&bytes[trailer_start..]);
+    let footer_len =
+        read_footer_length(footer_buffer).map_err(ArrowLayoutVerificationError::ArrowError)?;
 
     // read footer
-    let mut footer_data = vec![0; footer_len];
-    reader
-        .seek(SeekFrom::End(-10 - footer_len as i64))
-        .map_err(ArrowLayoutVerificationError::IOError)?;
-    reader
-        .read_exact(&mut footer_data)
-        .map_err(ArrowLayoutVerificationError::IOError)?;
+    let footer_data = &bytes[trailer_start - footer_len..trailer_start];
     let footer =
-        root_as_footer(&footer_data).map_err(ArrowLayoutVerificationError::InvalidFlatbuffer)?;
+        root_as_footer(footer_data).map_err(ArrowLayoutVerificationError::InvalidFlatbuffer)?;
 
+    Ok(footer)
+}
+
+fn read_record_batch_range(
+    footer: Footer<'_>,
+) -> Result<(&arrow::ipc::Block, usize, usize, usize), ArrowLayoutVerificationError> {
     // Read the record batch
     let record_batch_definitions = match footer.recordBatches() {
         Some(record_batch_definitions) => record_batch_definitions,
@@ -792,25 +796,32 @@ where
     }
 
     let record_batch_definition = record_batch_definitions.get(0);
-    let record_batch_len = record_batch_definition.bodyLength() as usize
-        + record_batch_definition.metaDataLength() as usize;
+    let record_batch_offset = record_batch_definition.offset() as usize;
     let record_batch_body_len = record_batch_definition.bodyLength() as usize;
+    let record_batch_len =
+        record_batch_body_len + record_batch_definition.metaDataLength() as usize;
 
-    // Read the actual record batch
-    let mut file_buffer = vec![0; record_batch_len];
-    match reader.seek(SeekFrom::Start(record_batch_definition.offset() as u64)) {
-        Ok(_) => {}
-        Err(e) => {
-            return Err(ArrowLayoutVerificationError::IOError(e));
-        }
-    }
-    match reader.read_exact(&mut file_buffer) {
-        Ok(_) => {}
-        Err(e) => {
-            return Err(ArrowLayoutVerificationError::IOError(e));
-        }
-    }
-    let buffer = Buffer::from(file_buffer);
+    Ok((
+        record_batch_definition,
+        record_batch_offset,
+        record_batch_body_len,
+        record_batch_len,
+    ))
+}
+
+/// Verifies that the buffers in the IPC file are 64 byte aligned
+/// and stored in Arrow in the way we expect.
+/// All non-benchmark test code should use this by loading the block
+/// with verification enabled.
+fn verify_buffers_layout(bytes: &[u8]) -> Result<(), ArrowLayoutVerificationError> {
+    // Read the IPC file and verify that the buffers are 64 byte aligned
+    // by inspecting the offsets, this is required since our
+    // size calculation assumes that the buffers are 64 byte aligned
+    // Space for ARROW_MAGIC (6 bytes) and length (4 bytes)
+    let footer = read_arrow_footer(bytes)?;
+    let (_, record_batch_offset, record_batch_body_len, record_batch_len) =
+        read_record_batch_range(footer)?;
+    let buffer = Buffer::from(&bytes[record_batch_offset..record_batch_offset + record_batch_len]);
 
     // This is borrowed from arrow-ipc parse_message.rs
     // https://arrow.apache.org/docs/format/Columnar.html#encapsulated-message-format
@@ -863,3 +874,30 @@ where
 
     Ok(())
 }
+
+#[cfg(test)]
+mod tests {
+
+    use std::sync::Arc;
+
+    use arrow::{
+        array::Int32Array,
+        datatypes::{DataType, Field, Schema},
+    };
+
+    use super::*;
+
+    #[test]
+    fn test_block_serde() {
+        let batch = RecordBatch::try_new(
+            Arc::new(Schema::new(vec![Field::new("id", DataType::Int32, false)])),
+            vec![Arc::new(Int32Array::from(vec![1, 2, 3, 4, 5]))],
+        )
+        .unwrap();
+        let b1 = Block::from_record_batch(Uuid::new_v4(), batch.clone());
+        let bytes = bincode::serialize(&b1).unwrap();
+        let b2 = bincode::deserialize::<Block>(&bytes).unwrap();
+        assert_eq!(b1.id, b2.id);
+        assert_eq!(b1.data.0, b2.data.0);
+    }
+}
