@@ -21,16 +21,17 @@ use std::{
 use thiserror::Error;
 use uuid::Uuid;
 
-pub(super) const CURRENT_VERSION: Version = Version::V1_1;
+pub(super) const CURRENT_VERSION: Version = Version::V1_2;
 
 // ================
 // Version
 // ================
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Serialize, Deserialize, PartialOrd, Ord)]
-pub(super) enum Version {
+pub(crate) enum Version {
     V1 = 1,
     V1_1 = 2,
+    V1_2 = 3,
 }
 
 impl Display for Version {
@@ -38,12 +39,13 @@ impl Display for Version {
         match self {
             Version::V1 => write!(f, "v1"),
             Version::V1_1 => write!(f, "v1.1"),
+            Version::V1_2 => write!(f, "v1.2"),
         }
     }
 }
 
 #[derive(Error, Debug)]
-pub(super) enum VersionError {
+pub enum VersionError {
     #[error("Unknown version: {0}")]
     UnknownVersion(String),
 }
@@ -60,6 +62,7 @@ impl TryFrom<&str> for Version {
         match s {
             "v1" => Ok(Version::V1),
             "v1.1" => Ok(Version::V1_1),
+            "v1.2" => Ok(Version::V1_2),
             _ => Err(VersionError::UnknownVersion(s.to_string())),
         }
     }
@@ -70,19 +73,29 @@ impl TryFrom<&str> for Version {
 // ================
 
 #[derive(Debug, Clone)]
-pub(super) struct RootWriter {
+pub struct RootWriter {
     pub(super) sparse_index: SparseIndexWriter,
     // Metadata
     pub(super) id: Uuid,
     pub(super) version: Version,
+    pub(super) prefix_path: String,
+    pub(super) max_block_size_bytes: usize,
 }
 
 impl RootWriter {
-    pub(super) fn new(version: Version, id: Uuid, sparse_index: SparseIndexWriter) -> Self {
+    pub(crate) fn new(
+        version: Version,
+        id: Uuid,
+        sparse_index: SparseIndexWriter,
+        prefix_path: String,
+        max_block_size_bytes: usize,
+    ) -> Self {
         Self {
             version,
             sparse_index,
             id,
+            prefix_path,
+            max_block_size_bytes,
         }
     }
 
@@ -178,11 +191,19 @@ impl RootWriter {
             data_arrays.push(built_counts);
         }
 
-        let metadata = HashMap::from_iter(vec![
+        let mut metadata = HashMap::from_iter(vec![
             ("version".to_string(), self.version.to_string()),
             ("id".to_string(), self.id.to_string()),
         ]);
 
+        if self.version >= Version::V1_2 {
+            // MIGRATION(06/25/2025 @sanket) -> V1.2 and above, we store the block size bytes in the metadata
+            metadata.insert(
+                "max_block_size_bytes".to_string(),
+                self.max_block_size_bytes.to_string(),
+            );
+        }
+
         let schema = Arc::new(Schema::new_with_metadata(schema_fields, metadata));
 
         let record_batch = match RecordBatch::try_new(schema, data_arrays) {
@@ -228,6 +249,8 @@ pub struct RootReader {
     // Metadata
     pub(super) id: Uuid,
     pub(super) version: Version,
+    pub(super) prefix_path: String,
+    pub(super) max_block_size_bytes: usize,
 }
 
 impl chroma_cache::Weighted for RootReader {
@@ -237,7 +260,7 @@ impl chroma_cache::Weighted for RootReader {
 }
 
 #[derive(Error, Debug)]
-pub(super) enum FromBytesError {
+pub enum FromBytesError {
     #[error("Error parsing UUID: {0}")]
     UuidParseError(#[from] uuid::Error),
     #[error("Error parsing version: {0}")]
@@ -252,6 +275,8 @@ pub(super) enum FromBytesError {
     IdMismatch,
     #[error(transparent)]
     VersionError(#[from] VersionError),
+    #[error("Error converting max block size bytes from string to usize")]
+    MaxBlockSizeBytesParseError,
 }
 
 impl ChromaError for FromBytesError {
@@ -264,14 +289,43 @@ impl ChromaError for FromBytesError {
             FromBytesError::NoDataError => chroma_error::ErrorCodes::Internal,
             FromBytesError::IdMismatch => chroma_error::ErrorCodes::InvalidArgument,
             FromBytesError::VersionError(e) => e.code(),
+            FromBytesError::MaxBlockSizeBytesParseError => chroma_error::ErrorCodes::Internal,
         }
     }
 }
 
 impl RootReader {
-    pub(super) fn from_bytes<'data, K: ArrowReadableKey<'data>>(
+    pub(super) fn get_all_block_ids_from_bytes(
         bytes: &[u8],
         id: Uuid,
+    ) -> Result<Vec<Uuid>, FromBytesError> {
+        let mut cursor = std::io::Cursor::new(bytes);
+        let arrow_reader = arrow::ipc::reader::FileReader::try_new(&mut cursor, None);
+
+        let record_batch = match arrow_reader {
+            Ok(mut reader) => match reader.next() {
+                Some(Ok(batch)) => batch,
+                Some(Err(e)) => return Err(FromBytesError::ArrowError(e)),
+                None => {
+                    return Err(FromBytesError::NoDataError);
+                }
+            },
+            Err(e) => return Err(FromBytesError::ArrowError(e)),
+        };
+
+        let (version, read_id) = Self::version_and_id_from_record_batch(&record_batch, id)?;
+        if read_id != id {
+            return Err(FromBytesError::IdMismatch);
+        }
+
+        Self::block_ids_from_record_batch(&record_batch, version)
+    }
+
+    pub(super) fn from_bytes<'data, K: ArrowReadableKey<'data>>(
+        bytes: &[u8],
+        prefix_path: &str,
+        id: Uuid,
+        default_max_block_size_bytes: usize,
     ) -> Result<Self, FromBytesError> {
         let mut cursor = std::io::Cursor::new(bytes);
         let arrow_reader = arrow::ipc::reader::FileReader::try_new(&mut cursor, None);
@@ -287,20 +341,12 @@ impl RootReader {
             Err(e) => return Err(FromBytesError::ArrowError(e)),
         };
 
-        let metadata = &record_batch.schema_ref().metadata;
-        let (version, read_id) = match (metadata.get("version"), metadata.get("id")) {
-            (Some(version), Some(read_id)) => (
-                Version::try_from(version.as_str())?,
-                Uuid::parse_str(read_id)?,
-            ),
-            (Some(_), None) => return Err(FromBytesError::MissingMetadata("id".to_string())),
-            (None, Some(_)) => {
-                return Err(FromBytesError::MissingMetadata("version".to_string()));
-            }
-            // We default to the current version in the absence of metadata for these fields for
-            // backwards compatibility
-            (None, None) => (Version::V1, id),
-        };
+        let (version, read_id) = Self::version_and_id_from_record_batch(&record_batch, id)?;
+        let max_block_size_bytes = Self::block_size_bytes_from_record_batch(
+            &record_batch,
+            version,
+            default_max_block_size_bytes,
+        )?;
 
         if read_id != id {
             return Err(FromBytesError::IdMismatch);
@@ -316,31 +362,7 @@ impl RootReader {
         // The sparse index copies the data so it can live as long as it needs to independently
         let record_batch: &'data RecordBatch = unsafe { std::mem::transmute(&record_batch) };
         let key_arr = record_batch.column(1);
-        let mut ids: Vec<uuid::Uuid> = Vec::new();
-        // Versions after V1 store uuid as bytes
-        if version == Version::V1 {
-            let id_array = record_batch
-                .column(2)
-                .as_any()
-                .downcast_ref::<StringArray>()
-                .expect("ID array to be a StringArray");
 
-            for i in 0..id_array.len() {
-                let id = Uuid::parse_str(id_array.value(i)).expect("ID to be a valid UUID");
-                ids.push(id);
-            }
-        } else {
-            let id_arr = record_batch
-                .column(2)
-                .as_any()
-                .downcast_ref::<BinaryArray>()
-                .expect("ID array to be a BinaryArray");
-
-            for i in 0..id_arr.len() {
-                let id = Uuid::from_slice(id_arr.value(i)).expect("ID to be a valid UUID");
-                ids.push(id);
-            }
-        }
         // Version 1.1 is the first version to have a count column
         let mut counts = None;
         if version >= Version::V1_1 {
@@ -352,6 +374,8 @@ impl RootReader {
             counts = Some(count_arr);
         }
 
+        let ids = Self::block_ids_from_record_batch(record_batch, version)?;
+
         let mut forward = BTreeMap::new();
         for (i, block_id) in ids.iter().enumerate() {
             let prefix = prefix_arr.value(i);
@@ -383,6 +407,8 @@ impl RootReader {
             version,
             sparse_index: sparse_index_reader,
             id,
+            prefix_path: prefix_path.to_string(),
+            max_block_size_bytes,
         })
     }
 
@@ -392,8 +418,86 @@ impl RootReader {
             version: self.version,
             sparse_index: new_sparse_index,
             id: new_id,
+            prefix_path: self.prefix_path.clone(),
+            max_block_size_bytes: self.max_block_size_bytes,
         }
     }
+
+    fn version_and_id_from_record_batch(
+        record_batch: &RecordBatch,
+        default_id: Uuid,
+    ) -> Result<(Version, Uuid), FromBytesError> {
+        let metadata = &record_batch.schema_ref().metadata;
+        match (metadata.get("version"), metadata.get("id")) {
+            (Some(version), Some(read_id)) => Ok((
+                Version::try_from(version.as_str())?,
+                Uuid::parse_str(read_id)?,
+            )),
+            (Some(_), None) => Err(FromBytesError::MissingMetadata("id".to_string())),
+            (None, Some(_)) => Err(FromBytesError::MissingMetadata("version".to_string())),
+            // We default to the current version in the absence of metadata for these fields for
+            // backwards compatibility
+            (None, None) => Ok((Version::V1, default_id)),
+        }
+    }
+
+    fn block_size_bytes_from_record_batch(
+        record_batch: &RecordBatch,
+        version: Version,
+        default_max_block_size_bytes: usize,
+    ) -> Result<usize, FromBytesError> {
+        let metadata = &record_batch.schema_ref().metadata;
+        match metadata.get("max_block_size_bytes") {
+            Some(size_str) => size_str
+                .parse::<usize>()
+                .map_err(|_| FromBytesError::MaxBlockSizeBytesParseError),
+            None => {
+                if version <= Version::V1_1 {
+                    // For versions V1 and V1.1, we do not store the max block size bytes
+                    // in the metadata, so we return the default value.
+                    Ok(default_max_block_size_bytes)
+                } else {
+                    // For version V1.2 and above, we expect the max block size bytes to be present
+                    Err(FromBytesError::MissingMetadata(
+                        "max_block_size_bytes".to_string(),
+                    ))
+                }
+            }
+        }
+    }
+
+    fn block_ids_from_record_batch(
+        record_batch: &RecordBatch,
+        version: Version,
+    ) -> Result<Vec<Uuid>, FromBytesError> {
+        let mut ids: Vec<uuid::Uuid> = Vec::new();
+        // Versions after V1 store uuid as bytes
+        if version == Version::V1 {
+            let id_array = record_batch
+                .column(2)
+                .as_any()
+                .downcast_ref::<StringArray>()
+                .expect("ID array to be a StringArray");
+
+            for i in 0..id_array.len() {
+                let id = Uuid::parse_str(id_array.value(i)).expect("ID to be a valid UUID");
+                ids.push(id);
+            }
+        } else {
+            let id_arr = record_batch
+                .column(2)
+                .as_any()
+                .downcast_ref::<BinaryArray>()
+                .expect("ID array to be a BinaryArray");
+
+            for i in 0..id_arr.len() {
+                let id = Uuid::from_slice(id_arr.value(i)).expect("ID to be a valid UUID");
+                ids.push(id);
+            }
+        }
+
+        Ok(ids)
+    }
 }
 
 #[cfg(test)]
@@ -411,7 +515,15 @@ mod test {
         let sparse_index = SparseIndexWriter::new(block_ids[0]);
 
         let bf_id = Uuid::new_v4();
-        let root_writer = RootWriter::new(CURRENT_VERSION, bf_id, sparse_index);
+        let prefix_path = "";
+        let max_block_size_bytes = 8 * 1024 * 1024; // 8 MiB
+        let root_writer = RootWriter::new(
+            CURRENT_VERSION,
+            bf_id,
+            sparse_index,
+            prefix_path.to_string(),
+            max_block_size_bytes,
+        );
 
         root_writer
             .sparse_index
@@ -446,8 +558,10 @@ mod test {
         let bytes = root_writer
             .to_bytes::<&str>()
             .expect("To be able to serialize");
+        // Try a different max_block_size_bytes but it should read from the metadata.
         let root_reader =
-            RootReader::from_bytes::<&str>(&bytes, bf_id).expect("To be able to deserialize");
+            RootReader::from_bytes::<&str>(&bytes, prefix_path, bf_id, 4 * 1024 * 1024)
+                .expect("To be able to deserialize");
 
         // Check that the sparse index is the same
         assert_eq!(
@@ -480,6 +594,112 @@ mod test {
 
         assert_eq!(root_writer.version, root_reader.version);
         assert_eq!(root_writer.id, root_reader.id);
+        assert_eq!(
+            root_writer.max_block_size_bytes,
+            root_reader.max_block_size_bytes
+        );
+    }
+
+    #[test]
+    fn test_to_from_bytes_fork() {
+        let block_ids = [
+            Uuid::new_v4(),
+            Uuid::new_v4(),
+            Uuid::new_v4(),
+            Uuid::new_v4(),
+        ];
+        let sparse_index = SparseIndexWriter::new(block_ids[0]);
+
+        let bf_id = Uuid::new_v4();
+        let prefix_path = "";
+        let max_block_size_bytes = 8 * 1024 * 1024; // 8 MiB
+        let root_writer = RootWriter::new(
+            CURRENT_VERSION,
+            bf_id,
+            sparse_index,
+            prefix_path.to_string(),
+            max_block_size_bytes,
+        );
+
+        root_writer
+            .sparse_index
+            .add_block(CompositeKey::new("prefix".to_string(), "a"), block_ids[1])
+            .expect("No error");
+        root_writer
+            .sparse_index
+            .add_block(CompositeKey::new("prefix".to_string(), "b"), block_ids[2])
+            .expect("No error");
+        root_writer
+            .sparse_index
+            .add_block(CompositeKey::new("prefix".to_string(), "c"), block_ids[3])
+            .expect("No error");
+
+        root_writer
+            .sparse_index
+            .set_count(block_ids[0], 1)
+            .expect("Set count should succeed");
+        root_writer
+            .sparse_index
+            .set_count(block_ids[1], 2)
+            .expect("Set count should succeed");
+        root_writer
+            .sparse_index
+            .set_count(block_ids[2], 3)
+            .expect("Set count should succeed");
+        root_writer
+            .sparse_index
+            .set_count(block_ids[3], 4)
+            .expect("Set count should succeed");
+
+        let bytes = root_writer
+            .to_bytes::<&str>()
+            .expect("To be able to serialize");
+        // Try a different max_block_size_bytes but it should read from the metadata.
+        let root_reader =
+            RootReader::from_bytes::<&str>(&bytes, prefix_path, bf_id, 4 * 1024 * 1024)
+                .expect("To be able to deserialize");
+        let fork_root_writer = root_reader.fork(Uuid::new_v4());
+
+        // Check that the sparse index is the same
+        assert_eq!(
+            root_writer.sparse_index.len(),
+            fork_root_writer.sparse_index.len()
+        );
+
+        // Check that the block mapping is the same
+        for (key, value) in root_writer.sparse_index.data.lock().forward.iter() {
+            assert_eq!(
+                fork_root_writer
+                    .sparse_index
+                    .data
+                    .lock()
+                    .forward
+                    .get(key)
+                    .unwrap(),
+                value
+            );
+        }
+
+        // Check that counts are the same
+        let writer_data = &root_writer.sparse_index.data.lock();
+        for (key, _) in writer_data.forward.iter() {
+            assert_eq!(
+                fork_root_writer
+                    .sparse_index
+                    .data
+                    .lock()
+                    .counts
+                    .get(key)
+                    .unwrap(),
+                writer_data.counts.get(key).unwrap()
+            );
+        }
+
+        assert_eq!(root_writer.version, fork_root_writer.version);
+        assert_eq!(
+            root_writer.max_block_size_bytes,
+            fork_root_writer.max_block_size_bytes
+        );
     }
 
     #[test]
@@ -498,9 +718,17 @@ mod test {
             CompositeKey::new("prefix".to_string(), "d"),
         ];
         let sparse_index = SparseIndexWriter::new(block_ids[0]);
+        let prefix_path = "";
 
         let bf_id = Uuid::new_v4();
-        let root_writer = RootWriter::new(Version::V1, bf_id, sparse_index);
+        let writer_max_block_size_bytes = 8 * 1024 * 1024; // 8 MiB
+        let root_writer = RootWriter::new(
+            Version::V1,
+            bf_id,
+            sparse_index,
+            prefix_path.to_string(),
+            writer_max_block_size_bytes,
+        );
         root_writer
             .sparse_index
             .set_count(block_ids[0], counts[0])
@@ -520,8 +748,10 @@ mod test {
             .to_bytes::<&str>()
             .expect("To be able to serialize");
 
+        let reader_block_size_bytes = 4 * 1024 * 1024; // 4 MiB
         let root_reader =
-            RootReader::from_bytes::<&str>(&bytes, bf_id).expect("To be able to deserialize");
+            RootReader::from_bytes::<&str>(&bytes, prefix_path, bf_id, reader_block_size_bytes)
+                .expect("To be able to deserialize");
 
         // Check the version is still v1
         assert_eq!(root_reader.version, Version::V1);
@@ -539,5 +769,6 @@ mod test {
                 0
             );
         }
+        assert_eq!(root_reader.max_block_size_bytes, reader_block_size_bytes);
     }
 }
