@@ -3,7 +3,7 @@ use std::mem::size_of;
 use chroma_types::{DataRecord, SpannPostingList};
 use roaring::RoaringBitmap;
 
-pub trait Value: Clone {
+pub trait Value: Clone + Send + Sync {
     fn get_size(&self) -> usize;
 }
 
@@ -31,6 +31,12 @@ impl Value for String {
     }
 }
 
+impl Value for f32 {
+    fn get_size(&self) -> usize {
+        4
+    }
+}
+
 impl Value for u32 {
     fn get_size(&self) -> usize {
         4
