@@ -1,19 +1,21 @@
 use super::{CacheError, Weighted};
+use ahash::RandomState;
 use chroma_error::ChromaError;
+use chroma_tracing::util::{StopWatchUnit, Stopwatch};
 use clap::Parser;
-use foyer::opentelemetry_0_27::OpenTelemetryMetricsRegistry;
 use foyer::{
     CacheBuilder, DirectFsDeviceOptions, Engine, FifoConfig, FifoPicker, HybridCacheBuilder,
-    InvalidRatioPicker, LargeEngineOptions, LfuConfig, LruConfig, RateLimitPicker, S3FifoConfig,
-    StorageKey, StorageValue, TracingOptions,
+    InvalidRatioPicker, LargeEngineOptions, LfuConfig, LruConfig, S3FifoConfig, StorageKey,
+    StorageValue, Throttle, TracingOptions,
 };
-use opentelemetry::global;
+use opentelemetry::{global, KeyValue};
 use serde::{Deserialize, Serialize};
+use std::fmt::Debug;
 use std::hash::Hash;
 use std::sync::Arc;
 use std::time::Duration;
 
-const MIB: usize = 1024 * 1024;
+pub const MIB: usize = 1024 * 1024;
 
 const fn default_capacity() -> usize {
     1048576
@@ -47,6 +49,10 @@ const fn default_recover_concurrency() -> usize {
     16
 }
 
+const fn default_deterministic_hashing() -> bool {
+    true
+}
+
 const fn default_admission_rate_limit() -> usize {
     100
 }
@@ -55,6 +61,12 @@ const fn default_shards() -> usize {
     64
 }
 
+const fn default_buffer_pool_size() -> usize {
+    // See https://github.com/foyer-rs/foyer/discussions/751
+    // This should be at least max_entry_size * flushers.
+    256
+}
+
 fn default_eviction() -> String {
     "lru".to_string()
 }
@@ -64,27 +76,36 @@ const fn default_invalid_ratio() -> f64 {
 }
 
 const fn default_trace_insert_us() -> usize {
-    1000 * 1000
+    1000 * 100
 }
 
 const fn default_trace_get_us() -> usize {
-    1000 * 1000
+    1000 * 100
 }
 
 const fn default_trace_obtain_us() -> usize {
-    1000 * 1000
+    1000 * 100
 }
 
 const fn default_trace_remove_us() -> usize {
-    1000 * 1000
+    1000 * 100
 }
 
 const fn default_trace_fetch_us() -> usize {
-    1000 * 1000
+    1000 * 100
+}
+
+fn default_name() -> String {
+    String::from("foyer")
 }
 
 #[derive(Deserialize, Debug, Clone, Serialize, Parser)]
 pub struct FoyerCacheConfig {
+    /// Name of the cache. All metrics for the cache are prefixed with name_.
+    #[arg(long, default_value = "foyer")]
+    #[serde(default = "default_name")]
+    pub name: String,
+
     /// Directory for disk cache data.
     #[arg(short, long)]
     pub dir: Option<String>,
@@ -114,6 +135,13 @@ pub struct FoyerCacheConfig {
     #[serde(default = "default_flushers")]
     pub flushers: usize,
 
+    /// Buffer pool size. (MiB)
+    /// This should be atleast max_entry_size * flushers.
+    /// See https://github.com/foyer-rs/foyer/discussions/751
+    #[arg(long, default_value_t = 256)]
+    #[serde(default = "default_buffer_pool_size")]
+    pub buffer_pool: usize,
+
     /// AKA fsync
     #[arg(long, default_value_t = false)]
     #[serde(default = "default_flush")]
@@ -129,6 +157,14 @@ pub struct FoyerCacheConfig {
     #[serde(default = "default_recover_concurrency")]
     pub recover_concurrency: usize,
 
+    /// Enable deterministic hashing.
+    /// If true, the cache will use a deterministic hasher which is stable
+    /// across restarts. Note that this hasher is not necessarily stable across
+    /// architectures or versions of foyer, and the underlying AHash.
+    #[arg(long, default_value_t = true)]
+    #[serde(default = "default_deterministic_hashing")]
+    pub deterministic_hashing: bool,
+
     /// Enable rated ticket admission picker if `admission_rate_limit > 0`. (MiB/s)
     #[arg(long, default_value_t = 100)]
     #[serde(default = "default_admission_rate_limit")]
@@ -150,27 +186,27 @@ pub struct FoyerCacheConfig {
     pub invalid_ratio: f64,
 
     /// Record insert trace threshold. Only effective with "mtrace" feature.
-    #[arg(long, default_value_t = 1000 * 1000)]
+    #[arg(long, default_value_t = 1000 * 100)]
     #[serde(default = "default_trace_insert_us")]
     pub trace_insert_us: usize,
 
     /// Record get trace threshold. Only effective with "mtrace" feature.
-    #[arg(long, default_value_t = 1000 * 1000)]
+    #[arg(long, default_value_t = 1000 * 100)]
     #[serde(default = "default_trace_get_us")]
     pub trace_get_us: usize,
 
     /// Record obtain trace threshold. Only effective with "mtrace" feature.
-    #[arg(long, default_value_t = 1000 * 1000)]
+    #[arg(long, default_value_t = 1000 * 100)]
     #[serde(default = "default_trace_obtain_us")]
     pub trace_obtain_us: usize,
 
     /// Record remove trace threshold. Only effective with "mtrace" feature.
-    #[arg(long, default_value_t = 1000 * 1000)]
+    #[arg(long, default_value_t = 1000 * 100)]
     #[serde(default = "default_trace_remove_us")]
     pub trace_remove_us: usize,
 
     /// Record fetch trace threshold. Only effective with "mtrace" feature.
-    #[arg(long, default_value_t = 1000 * 1000)]
+    #[arg(long, default_value_t = 1000 * 100)]
     #[serde(default = "default_trace_fetch_us")]
     pub trace_fetch_us: usize,
 }
@@ -187,6 +223,16 @@ impl FoyerCacheConfig {
         Ok(Box::new(FoyerHybridCache::hybrid(self).await?))
     }
 
+    pub async fn build_hybrid_test<K, V>(
+        &self,
+    ) -> Result<Box<FoyerHybridCache<K, V>>, Box<dyn ChromaError>>
+    where
+        K: Clone + Send + Sync + StorageKey + Eq + PartialEq + Hash + 'static,
+        V: Clone + Send + Sync + StorageValue + Weighted + 'static,
+    {
+        Ok(Box::new(FoyerHybridCache::hybrid(self).await?))
+    }
+
     /// Build an in-memory-only cache.
     pub async fn build_memory<K, V>(
         &self,
@@ -223,21 +269,31 @@ impl FoyerCacheConfig {
     }
 }
 
-struct Stopwatch<'a>(
-    &'a opentelemetry::metrics::Histogram<u64>,
-    std::time::Instant,
-);
-
-impl<'a> Stopwatch<'a> {
-    fn new(histogram: &'a opentelemetry::metrics::Histogram<u64>) -> Self {
-        Self(histogram, std::time::Instant::now())
-    }
-}
-
-impl Drop for Stopwatch<'_> {
-    fn drop(&mut self) {
-        let elapsed = self.1.elapsed().as_micros() as u64;
-        self.0.record(elapsed, &[]);
+impl Default for FoyerCacheConfig {
+    fn default() -> Self {
+        FoyerCacheConfig {
+            name: default_name(),
+            dir: None,
+            capacity: default_capacity(),
+            mem: default_mem(),
+            disk: default_disk(),
+            file_size: default_file_size(),
+            flushers: default_flushers(),
+            flush: default_flush(),
+            reclaimers: default_reclaimers(),
+            recover_concurrency: default_recover_concurrency(),
+            deterministic_hashing: default_deterministic_hashing(),
+            admission_rate_limit: default_admission_rate_limit(),
+            shards: default_shards(),
+            eviction: default_eviction(),
+            invalid_ratio: default_invalid_ratio(),
+            trace_insert_us: default_trace_insert_us(),
+            trace_get_us: default_trace_get_us(),
+            trace_obtain_us: default_trace_obtain_us(),
+            trace_remove_us: default_trace_remove_us(),
+            trace_fetch_us: default_trace_fetch_us(),
+            buffer_pool: default_buffer_pool_size(),
+        }
     }
 }
 
@@ -247,13 +303,25 @@ where
     K: Clone + Send + Sync + StorageKey + Eq + PartialEq + Hash + 'static,
     V: Clone + Send + Sync + StorageValue + Weighted + 'static,
 {
-    cache: foyer::HybridCache<K, V>,
+    cache: foyer::HybridCache<K, V, RandomState>,
     cache_hit: opentelemetry::metrics::Counter<u64>,
     cache_miss: opentelemetry::metrics::Counter<u64>,
     get_latency: opentelemetry::metrics::Histogram<u64>,
+    obtain_latency: opentelemetry::metrics::Histogram<u64>,
     insert_latency: opentelemetry::metrics::Histogram<u64>,
     remove_latency: opentelemetry::metrics::Histogram<u64>,
     clear_latency: opentelemetry::metrics::Histogram<u64>,
+    hostname: KeyValue,
+}
+
+impl<K, V> Debug for FoyerHybridCache<K, V>
+where
+    K: Clone + Send + Sync + StorageKey + Eq + PartialEq + Hash + 'static,
+    V: Clone + Send + Sync + StorageValue + Weighted + 'static,
+{
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("FoyerHybridCache").finish()
+    }
 }
 
 impl<K, V> FoyerHybridCache<K, V>
@@ -272,9 +340,16 @@ where
             .with_record_hybrid_remove_threshold(Duration::from_micros(config.trace_remove_us as _))
             .with_record_hybrid_fetch_threshold(Duration::from_micros(config.trace_fetch_us as _));
 
+        let otel_0_27_metrics = Box::new(
+            mixtrics::registry::opentelemetry_0_27::OpenTelemetryMetricsRegistry::new(
+                global::meter("chroma"),
+            ),
+        );
         let builder = HybridCacheBuilder::<K, V>::new()
-            .with_metrics_registry(OpenTelemetryMetricsRegistry::new(global::meter("chroma")))
-            .with_tracing_options(tracing_options)
+            .with_name(config.name.clone())
+            .with_metrics_registry(otel_0_27_metrics)
+            .with_tracing_options(tracing_options.clone())
+            .with_policy(foyer::HybridCachePolicy::WriteOnInsertion)
             .memory(config.mem)
             .with_shards(config.shards);
 
@@ -291,65 +366,86 @@ where
             }
         };
 
+        let builder = match config.deterministic_hashing {
+            true => {
+                // These are generated from a good RNG.
+                let rs = RandomState::with_seeds(
+                    18408126631592559320,
+                    14098607199905812554,
+                    3530350452151671086,
+                    4042281453092388365,
+                );
+                builder.with_hash_builder(rs)
+            }
+            false => builder.with_hash_builder(RandomState::new()),
+        };
+
         let Some(dir) = config.dir.as_ref() else {
             return Err(Box::new(CacheError::InvalidCacheConfig(
                 "missing dir".to_string(),
             )));
         };
 
-        let mut builder = builder
+        let mut device_options = DirectFsDeviceOptions::new(dir)
+            .with_capacity(config.disk * MIB)
+            .with_file_size(config.file_size * MIB);
+        if config.admission_rate_limit > 0 {
+            device_options = device_options.with_throttle(
+                Throttle::new().with_write_throughput(config.admission_rate_limit * MIB),
+            );
+        }
+
+        let builder = builder
             .with_weighter(|_, v| v.weight())
             .storage(Engine::Large)
-            .with_device_options(
-                DirectFsDeviceOptions::new(dir)
-                    .with_capacity(config.disk * MIB)
-                    .with_file_size(config.file_size * MIB),
-            )
+            .with_device_options(device_options)
             .with_flush(config.flush)
+            .with_recover_mode(foyer::RecoverMode::Strict)
             .with_large_object_disk_cache_options(
                 LargeEngineOptions::new()
                     .with_indexer_shards(config.shards)
                     .with_recover_concurrency(config.recover_concurrency)
                     .with_flushers(config.flushers)
                     .with_reclaimers(config.reclaimers)
+                    .with_buffer_pool_size(config.buffer_pool * MIB)
                     .with_eviction_pickers(vec![
                         Box::new(InvalidRatioPicker::new(config.invalid_ratio)),
                         Box::new(FifoPicker::default()),
                     ]),
             );
 
-        if config.admission_rate_limit > 0 {
-            builder = builder.with_admission_picker(Arc::new(RateLimitPicker::new(
-                config.admission_rate_limit * MIB,
-            )));
-        }
         let cache = builder.build().await.map_err(|e| {
-            Box::new(CacheError::InvalidCacheConfig(format!(
-                "builder failed: {:?}",
-                e
-            ))) as _
+            CacheError::InvalidCacheConfig(format!("builder failed: {:?}", e)).boxed()
         })?;
         cache.enable_tracing();
-        cache.update_tracing_options(
-            TracingOptions::new().with_record_hybrid_get_threshold(Duration::from_millis(10)),
-        );
+        cache.update_tracing_options(tracing_options);
         let meter = global::meter("chroma");
         let cache_hit = meter.u64_counter("cache_hit").build();
         let cache_miss = meter.u64_counter("cache_miss").build();
         let get_latency = meter.u64_histogram("get_latency").build();
+        let obtain_latency = meter.u64_histogram("obtain_latency").build();
         let insert_latency = meter.u64_histogram("insert_latency").build();
         let remove_latency = meter.u64_histogram("remove_latency").build();
         let clear_latency = meter.u64_histogram("clear_latency").build();
+        let hostname = std::env::var("HOSTNAME").unwrap_or("unknown".to_string());
+        let hostname_kv = KeyValue::new("hostname", hostname);
         Ok(FoyerHybridCache {
             cache,
             cache_hit,
             cache_miss,
             get_latency,
+            obtain_latency,
             insert_latency,
             remove_latency,
             clear_latency,
+            hostname: hostname_kv,
         })
     }
+
+    #[allow(dead_code)]
+    fn insert_to_disk(&self, key: K, value: V) {
+        self.cache.storage_writer(key).insert(value);
+    }
 }
 
 #[async_trait::async_trait]
@@ -358,35 +454,51 @@ where
     K: Clone + Send + Sync + StorageKey + Eq + PartialEq + Hash + 'static,
     V: Clone + Send + Sync + StorageValue + Weighted + 'static,
 {
-    #[tracing::instrument(skip(self, key))]
     async fn get(&self, key: &K) -> Result<Option<V>, CacheError> {
-        let _stopwatch = Stopwatch::new(&self.get_latency);
+        let hostname = &[self.hostname.clone()];
+        let _stopwatch = Stopwatch::new(&self.get_latency, hostname, StopWatchUnit::Millis);
         let res = self.cache.get(key).await?.map(|v| v.value().clone());
         if res.is_some() {
-            self.cache_hit.add(1, &[]);
+            self.cache_hit.add(1, hostname);
         } else {
-            self.cache_miss.add(1, &[]);
+            self.cache_miss.add(1, hostname);
         }
         Ok(res)
     }
 
-    #[tracing::instrument(skip(self, key, value))]
     async fn insert(&self, key: K, value: V) {
-        let _stopwatch = Stopwatch::new(&self.insert_latency);
+        let hostname = &[self.hostname.clone()];
+        let _stopwatch = Stopwatch::new(&self.insert_latency, hostname, StopWatchUnit::Millis);
         self.cache.insert(key, value);
     }
 
-    #[tracing::instrument(skip(self, key))]
     async fn remove(&self, key: &K) {
-        let _stopwatch = Stopwatch::new(&self.remove_latency);
+        let hostname = &[self.hostname.clone()];
+        let _stopwatch = Stopwatch::new(&self.remove_latency, hostname, StopWatchUnit::Millis);
         self.cache.remove(key);
     }
 
-    #[tracing::instrument(skip(self))]
     async fn clear(&self) -> Result<(), CacheError> {
-        let _stopwatch = Stopwatch::new(&self.clear_latency);
+        let hostname = &[self.hostname.clone()];
+        let _stopwatch = Stopwatch::new(&self.clear_latency, hostname, StopWatchUnit::Millis);
         Ok(self.cache.clear().await?)
     }
+
+    async fn obtain(&self, key: K) -> Result<Option<V>, CacheError> {
+        let hostname = &[self.hostname.clone()];
+        let _stopwatch = Stopwatch::new(&self.obtain_latency, hostname, StopWatchUnit::Millis);
+        let res = self.cache.obtain(key).await?.map(|v| v.value().clone());
+        if res.is_some() {
+            self.cache_hit.add(1, hostname);
+        } else {
+            self.cache_miss.add(1, hostname);
+        }
+        Ok(res)
+    }
+
+    async fn may_contain(&self, key: &K) -> bool {
+        self.cache.contains(key)
+    }
 }
 
 impl<K, V> super::PersistentCache<K, V> for FoyerHybridCache<K, V>
@@ -406,9 +518,21 @@ where
     cache_hit: opentelemetry::metrics::Counter<u64>,
     cache_miss: opentelemetry::metrics::Counter<u64>,
     get_latency: opentelemetry::metrics::Histogram<u64>,
+    obtain_latency: opentelemetry::metrics::Histogram<u64>,
     insert_latency: opentelemetry::metrics::Histogram<u64>,
     remove_latency: opentelemetry::metrics::Histogram<u64>,
     clear_latency: opentelemetry::metrics::Histogram<u64>,
+    hostname: KeyValue,
+}
+
+impl<K, V> Debug for FoyerPlainCache<K, V>
+where
+    K: Clone + Send + Sync + Eq + PartialEq + Hash + 'static,
+    V: Clone + Send + Sync + Weighted + 'static,
+{
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("FoyerPlainCache").finish()
+    }
 }
 
 impl<K, V> FoyerPlainCache<K, V>
@@ -421,6 +545,7 @@ where
         config: &FoyerCacheConfig,
     ) -> Result<FoyerPlainCache<K, V>, Box<dyn ChromaError>> {
         let cache = CacheBuilder::new(config.capacity)
+            .with_name(config.name.clone())
             .with_shards(config.shards)
             .with_weighter(|_: &_, v: &V| v.weight())
             .build();
@@ -428,17 +553,22 @@ where
         let cache_hit = meter.u64_counter("cache_hit").build();
         let cache_miss = meter.u64_counter("cache_miss").build();
         let get_latency = meter.u64_histogram("get_latency").build();
+        let obtain_latency = meter.u64_histogram("obtain_latency").build();
         let insert_latency = meter.u64_histogram("insert_latency").build();
         let remove_latency = meter.u64_histogram("remove_latency").build();
         let clear_latency = meter.u64_histogram("clear_latency").build();
+        let hostname = std::env::var("HOSTNAME").unwrap_or("unknown".to_string());
+        let hostname_kv = KeyValue::new("hostname", hostname);
         Ok(FoyerPlainCache {
             cache,
             cache_hit,
             cache_miss,
             get_latency,
+            obtain_latency,
             insert_latency,
             remove_latency,
             clear_latency,
+            hostname: hostname_kv,
         })
     }
 
@@ -472,6 +602,7 @@ where
         let evl = TokioEventListener(tx);
 
         let cache = CacheBuilder::new(config.capacity)
+            .with_name(config.name.clone())
             .with_shards(config.shards)
             .with_weighter(|_: &_, v: &V| v.weight())
             .with_event_listener(Arc::new(evl))
@@ -480,17 +611,22 @@ where
         let cache_hit = meter.u64_counter("cache_hit").build();
         let cache_miss = meter.u64_counter("cache_miss").build();
         let get_latency = meter.u64_histogram("get_latency").build();
+        let obtain_latency = meter.u64_histogram("obtain_latency").build();
         let insert_latency = meter.u64_histogram("insert_latency").build();
         let remove_latency = meter.u64_histogram("remove_latency").build();
         let clear_latency = meter.u64_histogram("clear_latency").build();
+        let hostname = std::env::var("HOSTNAME").unwrap_or("unknown".to_string());
+        let hostname_kv = KeyValue::new("hostname", hostname);
         Ok(FoyerPlainCache {
             cache,
             cache_hit,
             cache_miss,
             get_latency,
+            obtain_latency,
             insert_latency,
             remove_latency,
             clear_latency,
+            hostname: hostname_kv,
         })
     }
 }
@@ -501,36 +637,48 @@ where
     K: Clone + Send + Sync + Eq + PartialEq + Hash + 'static,
     V: Clone + Send + Sync + Weighted + 'static,
 {
-    #[tracing::instrument(skip(self, key))]
     async fn get(&self, key: &K) -> Result<Option<V>, CacheError> {
-        let _stopwatch = Stopwatch::new(&self.get_latency);
+        let hostname = &[self.hostname.clone()];
+        let _stopwatch = Stopwatch::new(&self.get_latency, hostname, StopWatchUnit::Millis);
         let res = self.cache.get(key).map(|v| v.value().clone());
         if res.is_some() {
-            self.cache_hit.add(1, &[]);
+            self.cache_hit.add(1, hostname);
         } else {
-            self.cache_miss.add(1, &[]);
+            self.cache_miss.add(1, hostname);
         }
         Ok(res)
     }
 
-    #[tracing::instrument(skip(self, key, value))]
     async fn insert(&self, key: K, value: V) {
-        let _stopwatch = Stopwatch::new(&self.insert_latency);
+        let hostname = &[self.hostname.clone()];
+        let _stopwatch = Stopwatch::new(&self.insert_latency, hostname, StopWatchUnit::Millis);
         self.cache.insert(key, value);
     }
 
-    #[tracing::instrument(skip(self, key))]
     async fn remove(&self, key: &K) {
-        let _stopwatch = Stopwatch::new(&self.remove_latency);
+        let hostname = &[self.hostname.clone()];
+        let _stopwatch = Stopwatch::new(&self.remove_latency, hostname, StopWatchUnit::Millis);
         self.cache.remove(key);
     }
 
-    #[tracing::instrument(skip(self))]
     async fn clear(&self) -> Result<(), CacheError> {
-        let _stopwatch = Stopwatch::new(&self.clear_latency);
+        let hostname = &[self.hostname.clone()];
+        let _stopwatch = Stopwatch::new(&self.clear_latency, hostname, StopWatchUnit::Millis);
         self.cache.clear();
         Ok(())
     }
+
+    async fn obtain(&self, key: K) -> Result<Option<V>, CacheError> {
+        let hostname = &[self.hostname.clone()];
+        let _stopwatch = Stopwatch::new(&self.obtain_latency, hostname, StopWatchUnit::Millis);
+        let res = self.cache.get(&key).map(|v| v.value().clone());
+        if res.is_some() {
+            self.cache_hit.add(1, hostname);
+        } else {
+            self.cache_miss.add(1, hostname);
+        }
+        Ok(res)
+    }
 }
 
 impl<K, V> super::PersistentCache<K, V> for FoyerPlainCache<K, V>
@@ -539,3 +687,180 @@ where
     V: Clone + Send + Sync + Weighted + StorageValue + 'static,
 {
 }
+
+#[cfg(test)]
+mod test {
+    use std::path::PathBuf;
+
+    use tokio::{fs::File, sync::mpsc};
+
+    use crate::Cache;
+
+    use super::*;
+
+    impl crate::Weighted for Arc<File> {
+        fn weight(&self) -> usize {
+            1
+        }
+    }
+
+    impl crate::Weighted for String {
+        fn weight(&self) -> usize {
+            self.len()
+        }
+    }
+
+    #[tokio::test]
+    async fn test_foyer_memory_cache_can_close_file_descriptor() {
+        let dir = tempfile::tempdir().expect("Should be able to create temp path");
+
+        let (tx, mut rx) = mpsc::unbounded_channel();
+        let fd_pool = FoyerCacheConfig {
+            capacity: 54,
+            ..Default::default()
+        }
+        .build_memory_with_event_listener::<PathBuf, Arc<File>>(tx)
+        .await
+        .expect("Should be able to build in memory cache");
+
+        tokio::spawn(async move { while rx.recv().await.is_some() {} });
+
+        for i in 0..10000 {
+            let path = dir.path().join(i.to_string());
+            let file = Arc::new(
+                File::create(path.as_path())
+                    .await
+                    .expect("Should be able to create new file descriptor"),
+            );
+            fd_pool.insert(path, file).await;
+        }
+    }
+
+    #[tokio::test]
+    async fn test_foyer_hybrid_cache_can_recover() {
+        let dir = tempfile::tempdir()
+            .expect("To be able to create temp path")
+            .path()
+            .to_str()
+            .expect("To be able to parse path")
+            .to_string();
+        let cache = FoyerCacheConfig {
+            dir: Some(dir.clone()),
+            ..Default::default()
+        }
+        .build_hybrid::<String, String>()
+        .await
+        .unwrap();
+
+        cache.insert("key1".to_string(), "value1".to_string()).await;
+
+        // Wait for flush to disk
+        tokio::time::sleep(std::time::Duration::from_secs(2)).await;
+        drop(cache);
+
+        // Test that we can recover the cache from disk.
+        let cache2 = FoyerCacheConfig {
+            dir: Some(dir.clone()),
+            ..FoyerCacheConfig::default()
+        }
+        .build_hybrid::<String, String>()
+        .await
+        .unwrap();
+
+        assert_eq!(
+            cache2.get(&"key1".to_string()).await.unwrap(),
+            Some("value1".to_string())
+        );
+
+        // Deterministic hashing off should not be able to recover the cache.
+        let cache3 = FoyerCacheConfig {
+            dir: Some(dir.clone()),
+            deterministic_hashing: false,
+            ..FoyerCacheConfig::default()
+        }
+        .build_hybrid::<String, String>()
+        .await
+        .unwrap();
+
+        assert_eq!(cache3.get(&"key1".to_string()).await.unwrap(), None);
+    }
+
+    #[tokio::test]
+    async fn test_writing_only_to_disk_works() {
+        let dir = tempfile::tempdir()
+            .expect("To be able to create temp path")
+            .path()
+            .to_str()
+            .expect("To be able to parse path")
+            .to_string();
+        let cache = FoyerCacheConfig {
+            dir: Some(dir.clone()),
+            flush: true,
+            ..Default::default()
+        }
+        .build_hybrid_test::<String, String>()
+        .await
+        .unwrap();
+        // Insert a 512KB string value generated at random by passing memory.
+        let large_value = "val1".repeat(512 * 1024);
+        cache.insert_to_disk("key1".to_string(), large_value.clone());
+        // Sleep for 2 secs.
+        // This should give the cache enough time to flush the data to disk.
+        tokio::time::sleep(std::time::Duration::from_secs(2)).await;
+        let value = cache
+            .get(&"key1".to_string())
+            .await
+            .expect("Expected to be able to get value")
+            .expect("Value should not be None");
+        assert_eq!(value, large_value);
+    }
+
+    #[tokio::test]
+    async fn test_inserted_key_immediately_available() {
+        let dir = tempfile::tempdir()
+            .expect("To be able to create temp path")
+            .path()
+            .to_str()
+            .expect("To be able to parse path")
+            .to_string();
+        let cache = FoyerCacheConfig {
+            dir: Some(dir.clone()),
+            flush: true,
+            ..Default::default()
+        }
+        .build_hybrid_test::<String, String>()
+        .await
+        .unwrap();
+
+        cache.insert("key1".to_string(), "foo".to_string()).await;
+
+        let value = cache
+            .get(&"key1".to_string())
+            .await
+            .expect("Expected to be able to get value")
+            .expect("Value should not be None");
+        assert_eq!(value, "foo");
+    }
+
+    #[tokio::test]
+    async fn test_may_contain() {
+        let dir = tempfile::tempdir()
+            .expect("To be able to create temp path")
+            .path()
+            .to_str()
+            .expect("To be able to parse path")
+            .to_string();
+        let cache = FoyerCacheConfig {
+            dir: Some(dir.clone()),
+            flush: true,
+            ..Default::default()
+        }
+        .build_hybrid_test::<String, String>()
+        .await
+        .unwrap();
+
+        cache.insert("key1".to_string(), "foo".to_string()).await;
+        assert!(cache.may_contain(&"key1".to_string()).await);
+        assert!(!cache.may_contain(&"key2".to_string()).await);
+    }
+}
