@@ -201,6 +201,7 @@ copyright Qdrant, licensed under the Apache 2.0 license.
 */
 
 use chroma_error::{ChromaError, ErrorCodes};
+use chroma_types::Space;
 use thiserror::Error;
 
 /// The distance function enum.
@@ -219,6 +220,16 @@ pub enum DistanceFunction {
     InnerProduct,
 }
 
+impl From<Space> for DistanceFunction {
+    fn from(space: Space) -> Self {
+        match space {
+            Space::L2 => DistanceFunction::Euclidean,
+            Space::Cosine => DistanceFunction::Cosine,
+            Space::Ip => DistanceFunction::InnerProduct,
+        }
+    }
+}
+
 impl DistanceFunction {
     // TOOD: Should we error if mismatched dimensions?
     pub fn distance(&self, a: &[f32], b: &[f32]) -> f32 {
@@ -233,12 +244,23 @@ impl DistanceFunction {
                     }
                 }
                 #[cfg(all(
-                    any(target_arch = "x86_64", target_arch = "x86"),
-                    target_feature = "sse"
+                    target_arch = "x86_64",
+                    all(
+                        target_feature = "avx512f",
+                        target_feature = "avx512dq",
+                        target_feature = "avx512bw",
+                        target_feature = "avx512vl",
+                        target_feature = "fma"
+                    )
                 ))]
                 {
-                    if std::arch::is_x86_feature_detected!("sse") {
-                        return unsafe { crate::distance_sse::euclidean_distance(a, b) };
+                    if std::arch::is_x86_feature_detected!("avx512f")
+                        && std::arch::is_x86_feature_detected!("avx512dq")
+                        && std::arch::is_x86_feature_detected!("avx512bw")
+                        && std::arch::is_x86_feature_detected!("avx512vl")
+                        && std::arch::is_x86_feature_detected!("fma")
+                    {
+                        return unsafe { crate::distance_avx512::euclidean_distance(a, b) };
                     }
                 }
                 #[cfg(all(
@@ -252,6 +274,15 @@ impl DistanceFunction {
                         return unsafe { crate::distance_avx::euclidean_distance(a, b) };
                     }
                 }
+                #[cfg(all(
+                    any(target_arch = "x86_64", target_arch = "x86"),
+                    target_feature = "sse"
+                ))]
+                {
+                    if std::arch::is_x86_feature_detected!("sse") {
+                        return unsafe { crate::distance_sse::euclidean_distance(a, b) };
+                    }
+                }
                 let mut sum = 0.0;
                 for i in 0..a.len() {
                     sum += (a[i] - b[i]).powi(2);
@@ -266,12 +297,23 @@ impl DistanceFunction {
                     }
                 }
                 #[cfg(all(
-                    any(target_arch = "x86_64", target_arch = "x86"),
-                    target_feature = "sse"
+                    target_arch = "x86_64",
+                    all(
+                        target_feature = "avx512f",
+                        target_feature = "avx512dq",
+                        target_feature = "avx512bw",
+                        target_feature = "avx512vl",
+                        target_feature = "fma"
+                    )
                 ))]
                 {
-                    if std::arch::is_x86_feature_detected!("sse") {
-                        return unsafe { crate::distance_sse::cosine_distance(a, b) };
+                    if std::arch::is_x86_feature_detected!("avx512f")
+                        && std::arch::is_x86_feature_detected!("avx512dq")
+                        && std::arch::is_x86_feature_detected!("avx512bw")
+                        && std::arch::is_x86_feature_detected!("avx512vl")
+                        && std::arch::is_x86_feature_detected!("fma")
+                    {
+                        return unsafe { crate::distance_avx512::cosine_distance(a, b) };
                     }
                 }
                 #[cfg(all(
@@ -285,6 +327,15 @@ impl DistanceFunction {
                         return unsafe { crate::distance_avx::cosine_distance(a, b) };
                     }
                 }
+                #[cfg(all(
+                    any(target_arch = "x86_64", target_arch = "x86"),
+                    target_feature = "sse"
+                ))]
+                {
+                    if std::arch::is_x86_feature_detected!("sse") {
+                        return unsafe { crate::distance_sse::cosine_distance(a, b) };
+                    }
+                }
                 // For cosine we just assume the vectors have been normalized, since that
                 // is what our indices expect.
                 let mut sum = 0.0;
@@ -301,12 +352,23 @@ impl DistanceFunction {
                     }
                 }
                 #[cfg(all(
-                    any(target_arch = "x86_64", target_arch = "x86"),
-                    target_feature = "sse"
+                    target_arch = "x86_64",
+                    all(
+                        target_feature = "avx512f",
+                        target_feature = "avx512dq",
+                        target_feature = "avx512bw",
+                        target_feature = "avx512vl",
+                        target_feature = "fma"
+                    )
                 ))]
                 {
-                    if std::arch::is_x86_feature_detected!("sse") {
-                        return unsafe { crate::distance_sse::inner_product(a, b) };
+                    if std::arch::is_x86_feature_detected!("avx512f")
+                        && std::arch::is_x86_feature_detected!("avx512dq")
+                        && std::arch::is_x86_feature_detected!("avx512bw")
+                        && std::arch::is_x86_feature_detected!("avx512vl")
+                        && std::arch::is_x86_feature_detected!("fma")
+                    {
+                        return unsafe { crate::distance_avx512::inner_product(a, b) };
                     }
                 }
                 #[cfg(all(
@@ -320,6 +382,15 @@ impl DistanceFunction {
                         return unsafe { crate::distance_avx::inner_product(a, b) };
                     }
                 }
+                #[cfg(all(
+                    any(target_arch = "x86_64", target_arch = "x86"),
+                    target_feature = "sse"
+                ))]
+                {
+                    if std::arch::is_x86_feature_detected!("sse") {
+                        return unsafe { crate::distance_sse::inner_product(a, b) };
+                    }
+                }
                 let mut sum = 0.0;
                 for i in 0..a.len() {
                     sum += a[i] * b[i];
