@@ -27,9 +27,7 @@ pub enum ConversionError {
 
 impl ChromaError for ConversionError {
     fn code(&self) -> ErrorCodes {
-        match self {
-            ConversionError::DecodeError => ErrorCodes::InvalidArgument,
-        }
+        ErrorCodes::InvalidArgument
     }
 }
 
@@ -38,3 +36,16 @@ impl From<ConversionError> for Status {
         Status::invalid_argument(value.to_string())
     }
 }
+
+#[derive(thiserror::Error, Debug)]
+#[error(transparent)]
+pub enum WrappedSerdeJsonError {
+    #[error(transparent)]
+    SerdeJsonError(#[from] serde_json::Error),
+}
+
+impl ChromaError for WrappedSerdeJsonError {
+    fn code(&self) -> ErrorCodes {
+        ErrorCodes::InvalidArgument
+    }
+}
