@@ -2,7 +2,11 @@ mod compaction_manager;
 pub(crate) mod config;
 mod scheduler;
 mod scheduler_policy;
+mod tasks;
 mod types;
 
 pub(crate) use compaction_manager::*;
 pub(crate) use types::*;
+
+pub mod compaction_client;
+pub mod compaction_server;
