@@ -1,32 +1,107 @@
-use std::collections::HashSet;
+use std::collections::{HashMap, HashSet};
 use std::str::FromStr;
+use std::sync::Arc;
+use std::time::{Duration, SystemTime};
 
+use chroma_config::assignment::assignment_policy::AssignmentPolicy;
+use chroma_log::{CollectionInfo, CollectionRecord, Log};
+use chroma_memberlist::memberlist_provider::Memberlist;
+use chroma_storage::Storage;
+use chroma_sysdb::{GetCollectionsOptions, SysDb};
 use chroma_types::CollectionUuid;
 use figment::providers::Env;
 use figment::Figment;
+use s3heap_service::SysDbScheduler;
 use serde::Deserialize;
 use uuid::Uuid;
 
-use crate::assignment::assignment_policy::AssignmentPolicy;
 use crate::compactor::scheduler_policy::SchedulerPolicy;
+use crate::compactor::tasks::TaskHeapReader;
 use crate::compactor::types::CompactionJob;
-use crate::log::log::CollectionInfo;
-use crate::log::log::CollectionRecord;
-use crate::log::log::Log;
-use crate::memberlist::Memberlist;
-use crate::sysdb::sysdb::SysDb;
+
+#[derive(Debug, Clone)]
+pub(crate) struct SchedulerMetrics {
+    dead_jobs_count: opentelemetry::metrics::Gauge<u64>,
+}
+
+impl Default for SchedulerMetrics {
+    fn default() -> Self {
+        let meter = opentelemetry::global::meter("chroma_compactor");
+        let dead_jobs_count = meter
+            .u64_gauge("compactor_dead_jobs_count")
+            .with_description("Number of collections with failed jobs")
+            .build();
+
+        Self { dead_jobs_count }
+    }
+}
+
+impl SchedulerMetrics {
+    fn update_dead_jobs_count(&self, count: usize) {
+        // Create a callback that will be called when metrics are collected
+        self.dead_jobs_count.record(count.try_into().unwrap(), &[]);
+    }
+}
+
+#[derive(Debug)]
+struct FailedJob {
+    failure_count: u8,
+}
+
+impl FailedJob {
+    fn new() -> Self {
+        Self { failure_count: 1 }
+    }
+
+    fn increment_failure(&mut self, max_failure_count: u8) {
+        if self.failure_count >= max_failure_count {
+            return;
+        }
+        self.failure_count += 1;
+    }
+
+    fn failure_count(&self) -> u8 {
+        self.failure_count
+    }
+}
+
+struct InProgressJob {
+    expires_at: SystemTime,
+}
+
+impl InProgressJob {
+    fn new(job_expiry_seconds: u64) -> Self {
+        Self {
+            expires_at: SystemTime::now() + Duration::from_secs(job_expiry_seconds),
+        }
+    }
+
+    fn is_expired(&self) -> bool {
+        SystemTime::now() >= self.expires_at
+    }
+}
 
 pub(crate) struct Scheduler {
-    my_ip: String,
-    log: Box<Log>,
-    sysdb: Box<SysDb>,
+    my_member_id: String,
+    log: Log,
+    sysdb: SysDb,
     policy: Box<dyn SchedulerPolicy>,
     job_queue: Vec<CompactionJob>,
     max_concurrent_jobs: usize,
     min_compaction_size: usize,
     memberlist: Option<Memberlist>,
     assignment_policy: Box<dyn AssignmentPolicy>,
+    oneoff_collections: HashSet<CollectionUuid>,
     disabled_collections: HashSet<CollectionUuid>,
+    deleted_collections: HashSet<CollectionUuid>,
+    collections_needing_repair: HashMap<CollectionUuid, i64>,
+    in_progress_jobs: HashMap<CollectionUuid, InProgressJob>,
+    job_expiry_seconds: u64,
+    failing_jobs: HashMap<CollectionUuid, FailedJob>,
+    dead_jobs: HashSet<CollectionUuid>,
+    max_failure_count: u8,
+    metrics: SchedulerMetrics,
+    tasks: TaskHeapReader,
 }
 
 #[derive(Deserialize, Debug)]
@@ -38,16 +113,23 @@ impl Scheduler {
     #[allow(clippy::too_many_arguments)]
     pub(crate) fn new(
         my_ip: String,
-        log: Box<Log>,
-        sysdb: Box<SysDb>,
+        log: Log,
+        sysdb: SysDb,
+        storage: Storage,
         policy: Box<dyn SchedulerPolicy>,
         max_concurrent_jobs: usize,
         min_compaction_size: usize,
         assignment_policy: Box<dyn AssignmentPolicy>,
         disabled_collections: HashSet<CollectionUuid>,
+        job_expiry_seconds: u64,
+        max_failure_count: u8,
     ) -> Scheduler {
+        let heap_scheduler =
+            Arc::new(SysDbScheduler::new(sysdb.clone())) as Arc<dyn s3heap::HeapScheduler>;
+        let tasks = TaskHeapReader::new(storage, heap_scheduler);
+
         Scheduler {
-            my_ip,
+            my_member_id: my_ip,
             log,
             sysdb,
             min_compaction_size,
@@ -56,10 +138,45 @@ impl Scheduler {
             max_concurrent_jobs,
             memberlist: None,
             assignment_policy,
+            oneoff_collections: HashSet::new(),
             disabled_collections,
+            deleted_collections: HashSet::new(),
+            collections_needing_repair: HashMap::new(),
+            in_progress_jobs: HashMap::new(),
+            job_expiry_seconds,
+            failing_jobs: HashMap::new(),
+            max_failure_count,
+            dead_jobs: HashSet::new(),
+            metrics: SchedulerMetrics::default(),
+            tasks,
         }
     }
 
+    pub(crate) fn add_oneoff_collections(&mut self, ids: Vec<CollectionUuid>) {
+        self.oneoff_collections.extend(ids);
+    }
+
+    pub(crate) fn get_oneoff_collections(&self) -> Vec<CollectionUuid> {
+        self.oneoff_collections.iter().cloned().collect()
+    }
+
+    pub(crate) fn drain_deleted_collections(&mut self) -> Vec<CollectionUuid> {
+        self.deleted_collections.drain().collect()
+    }
+
+    pub(crate) fn drain_collections_requiring_repair(&mut self) -> Vec<(CollectionUuid, i64)> {
+        self.collections_needing_repair.drain().collect()
+    }
+
+    pub(crate) fn require_repair(&mut self, collection_id: CollectionUuid, offset_in_sysdb: i64) {
+        self.collections_needing_repair
+            .insert(collection_id, offset_in_sysdb);
+    }
+
+    pub(crate) fn get_dead_jobs(&self) -> Vec<CollectionUuid> {
+        self.dead_jobs.iter().cloned().collect()
+    }
+
     async fn get_collections_with_new_data(&mut self) -> Vec<CollectionInfo> {
         let collections = self
             .log
@@ -67,7 +184,10 @@ impl Scheduler {
             .await;
 
         match collections {
-            Ok(collections) => collections,
+            Ok(collections) => {
+                tracing::info!("Collections with new data: {collections:?}");
+                collections
+            }
             Err(e) => {
                 tracing::error!("Error: {:?}", e);
                 Vec::new()
@@ -81,9 +201,25 @@ impl Scheduler {
     ) -> Vec<CollectionRecord> {
         let mut collection_records = Vec::new();
         for collection_info in collections {
+            let failure_count = self
+                .failing_jobs
+                .get(&collection_info.collection_id)
+                .map(|job| job.failure_count())
+                .unwrap_or(0);
+
+            if failure_count >= self.max_failure_count {
+                tracing::warn!(
+                    "Job for collection {} failed more than {} times, moving this to dead jobs and skipping compaction for it",
+                    collection_info.collection_id,
+                    self.max_failure_count
+                );
+                self.kill_collection(collection_info.collection_id);
+                continue;
+            }
             if self
                 .disabled_collections
                 .contains(&collection_info.collection_id)
+                || self.dead_jobs.contains(&collection_info.collection_id)
             {
                 tracing::info!(
                     "Ignoring collection: {:?} because it disabled for compaction",
@@ -91,30 +227,39 @@ impl Scheduler {
                 );
                 continue;
             }
-            let collection_id = Some(collection_info.collection_id);
             // TODO: add a cache to avoid fetching the same collection multiple times
             let result = self
                 .sysdb
-                .get_collections(collection_id, None, None, None)
+                .get_collections(GetCollectionsOptions {
+                    collection_id: Some(collection_info.collection_id),
+                    ..Default::default()
+                })
                 .await;
 
             match result {
                 Ok(collection) => {
                     if collection.is_empty() {
-                        tracing::error!(
-                            "Collection not found: {:?}",
-                            collection_info.collection_id
-                        );
+                        self.deleted_collections
+                            .insert(collection_info.collection_id);
                         continue;
                     }
 
                     // TODO: make querying the last compaction time in batch
-                    let log_position_in_collecion = collection[0].log_position;
+                    let log_position_in_collection = collection[0].log_position;
                     let tenant_ids = vec![collection[0].tenant.clone()];
                     let tenant = self.sysdb.get_last_compaction_time(tenant_ids).await;
 
                     let last_compaction_time = match tenant {
-                        Ok(tenant) => tenant[0].last_compaction_time,
+                        Ok(tenant) => {
+                            if tenant.is_empty() {
+                                tracing::info!(
+                                    "Ignoring collection: {:?}",
+                                    collection_info.collection_id
+                                );
+                                continue;
+                            }
+                            tenant[0].last_compaction_time
+                        }
                         Err(e) => {
                             tracing::error!("Error: {:?}", e);
                             // Ignore this collection id for this compaction iteration
@@ -130,14 +275,17 @@ impl Scheduler {
                     // offset in log is the first offset in the log that has not been compacted. Note that
                     // since the offset is the first offset of log we get from the log service, we should
                     // use this offset to pull data from the log service.
-                    if log_position_in_collecion + 1 < offset {
+                    if log_position_in_collection + 1 < offset {
                         panic!(
-                            "offset in sysdb is less than offset in log, this should not happen!"
+                            "offset in sysdb ({}) is less than offset in log ({}) for {}",
+                            log_position_in_collection + 1,
+                            offset,
+                            collection[0].collection_id,
                         )
                     } else {
                         // The offset in sysdb is the last offset that has been compacted.
                         // We need to start from the next offset.
-                        offset = log_position_in_collecion + 1;
+                        offset = log_position_in_collection + 1;
                     }
 
                     collection_records.push(CollectionRecord {
@@ -147,6 +295,7 @@ impl Scheduler {
                         first_record_time: collection_info.first_log_ts,
                         offset,
                         collection_version: collection[0].version,
+                        collection_logical_size_bytes: collection[0].size_bytes_post_compaction,
                     });
                 }
                 Err(e) => {
@@ -154,21 +303,27 @@ impl Scheduler {
                 }
             }
         }
-        self.filter_collections(collection_records)
+        collection_records
     }
 
     fn filter_collections(&mut self, collections: Vec<CollectionRecord>) -> Vec<CollectionRecord> {
         let mut filtered_collections = Vec::new();
         let members = self.memberlist.as_ref().unwrap();
-        self.assignment_policy.set_members(members.clone());
+        let members_as_string = members
+            .iter()
+            .map(|member| member.member_id.clone())
+            .collect();
+        self.assignment_policy.set_members(members_as_string);
+
         for collection in collections {
             let result = self
                 .assignment_policy
                 // NOTE(rescrv):  Need to use the untyped uuid here.
-                .assign(collection.collection_id.0.to_string().as_str());
+                .assign_one(collection.collection_id.0.to_string().as_str());
+
             match result {
                 Ok(member) => {
-                    if member == self.my_ip {
+                    if member == self.my_member_id {
                         filtered_collections.push(collection);
                     }
                 }
@@ -182,11 +337,120 @@ impl Scheduler {
     }
 
     pub(crate) async fn schedule_internal(&mut self, collection_records: Vec<CollectionRecord>) {
-        let jobs = self
-            .policy
-            .determine(collection_records, self.max_concurrent_jobs as i32);
         self.job_queue.clear();
-        self.job_queue.extend(jobs);
+        let mut scheduled_collections = Vec::new();
+        for record in collection_records {
+            if self.is_job_in_progress(&record.collection_id) {
+                tracing::info!(
+                    "Compaction for {} is already in progress, skipping",
+                    record.collection_id
+                );
+                continue;
+            }
+            if self.oneoff_collections.contains(&record.collection_id) {
+                tracing::info!(
+                    "Creating one-off compaction job for collection: {}",
+                    record.collection_version
+                );
+                self.job_queue.push(CompactionJob {
+                    collection_id: record.collection_id,
+                });
+                self.oneoff_collections.remove(&record.collection_id);
+                if self.job_queue.len() == self.max_concurrent_jobs {
+                    return;
+                }
+            } else {
+                if self.in_progress_jobs.len() >= self.max_concurrent_jobs {
+                    tracing::info!(
+                        "Max concurrent jobs reached, skipping compaction for {}",
+                        record.collection_id
+                    );
+                    return;
+                }
+                scheduled_collections.push(record);
+            }
+        }
+
+        let filtered_collections = self.filter_collections(scheduled_collections);
+        self.job_queue.extend(
+            self.policy
+                .determine(filtered_collections, self.max_concurrent_jobs as i32),
+        );
+        self.job_queue
+            .truncate(self.max_concurrent_jobs - self.in_progress_jobs.len());
+
+        // At this point, nobody should modify the job queue and every collection
+        // in the job queue will definitely be compacted. It is now safe to add
+        // them to the in-progress set.
+        let job_ids: Vec<_> = self.job_queue.iter().map(|j| j.collection_id).collect();
+        for collection_id in job_ids {
+            self.add_in_progress(collection_id);
+        }
+    }
+
+    fn is_job_in_progress(&mut self, collection_id: &CollectionUuid) -> bool {
+        match self.in_progress_jobs.get(collection_id) {
+            Some(job) if job.is_expired() => {
+                tracing::info!(
+                    "Compaction for {} is expired, removing from dedup set.",
+                    collection_id
+                );
+                self.fail_collection(*collection_id);
+                false
+            }
+            Some(_) => true,
+            None => false,
+        }
+    }
+
+    fn add_in_progress(&mut self, collection_id: CollectionUuid) {
+        self.in_progress_jobs
+            .insert(collection_id, InProgressJob::new(self.job_expiry_seconds));
+    }
+
+    pub(crate) fn succeed_collection(&mut self, collection_id: CollectionUuid) {
+        if self.in_progress_jobs.remove(&collection_id).is_none() {
+            tracing::warn!(
+                "Expired compaction for {} just successfully finished.",
+                collection_id
+            );
+            return;
+        }
+        self.failing_jobs.remove(&collection_id);
+    }
+
+    pub(crate) fn fail_collection(&mut self, collection_id: CollectionUuid) {
+        if self.in_progress_jobs.remove(&collection_id).is_none() {
+            tracing::warn!(
+                "Expired compaction for {} just unsuccessfully finished.",
+                collection_id
+            );
+            return;
+        }
+        match self.failing_jobs.get_mut(&collection_id) {
+            Some(failed_job) => {
+                failed_job.increment_failure(self.max_failure_count);
+                tracing::warn!(
+                    "Job for collection {} failed {}/{} times",
+                    collection_id,
+                    failed_job.failure_count(),
+                    self.max_failure_count
+                );
+            }
+            None => {
+                self.failing_jobs.insert(collection_id, FailedJob::new());
+                tracing::warn!(
+                    "Job for collection {} failed for the first time",
+                    collection_id
+                );
+            }
+        }
+    }
+
+    pub(crate) fn kill_collection(&mut self, collection_id: CollectionUuid) {
+        self.failing_jobs.remove(&collection_id);
+        self.dead_jobs.insert(collection_id);
+        self.metrics.update_dead_jobs_count(self.dead_jobs.len());
     }
 
     pub(crate) fn recompute_disabled_collections(&mut self) {
@@ -223,6 +487,21 @@ impl Scheduler {
         // Recompute disabled list.
         self.recompute_disabled_collections();
         let collections = self.get_collections_with_new_data().await;
+        let tasks = self
+            .tasks
+            .get_tasks_scheduled_for_execution(
+                s3heap::Limits::default().with_items(self.max_concurrent_jobs),
+            )
+            .await;
+        for task in tasks {
+            tracing::info!(
+                "SCHEDULING TASKS FOR {:?} {:?} {:?} {:?}",
+                task.bucket,
+                task.collection_id,
+                task.task_id,
+                task.nonce,
+            );
+        }
         if collections.is_empty() {
             return;
         }
@@ -237,6 +516,10 @@ impl Scheduler {
     pub(crate) fn set_memberlist(&mut self, memberlist: Memberlist) {
         self.memberlist = Some(memberlist);
     }
+
+    pub(crate) fn has_memberlist(&self) -> bool {
+        self.memberlist.is_some()
+    }
 }
 
 #[cfg(test)]
@@ -244,23 +527,36 @@ mod tests {
     use std::str::FromStr;
 
     use super::*;
-    use crate::assignment::assignment_policy::RendezvousHashingAssignmentPolicy;
     use crate::compactor::scheduler_policy::LasCompactionTimeSchedulerPolicy;
-    use crate::log::log::InMemoryLog;
-    use crate::log::log::InternalLogRecord;
-    use crate::sysdb::test_sysdb::TestSysDb;
-    use chroma_types::{Collection, CollectionUuid, LogRecord, Operation, OperationRecord};
+    use chroma_config::assignment::assignment_policy::RendezvousHashingAssignmentPolicy;
+    use chroma_log::in_memory_log::{InMemoryLog, InternalLogRecord};
+    use chroma_memberlist::memberlist_provider::Member;
+    use chroma_storage::s3_client_for_test_with_new_bucket;
+    use chroma_sysdb::TestSysDb;
+    use chroma_types::{Collection, LogRecord, Operation, OperationRecord};
 
     #[tokio::test]
-    async fn test_scheduler() {
-        let mut log = Box::new(Log::InMemory(InMemoryLog::new()));
-        let in_memory_log = match *log {
+    async fn test_k8s_integration_scheduler() {
+        let storage = s3_client_for_test_with_new_bucket().await;
+
+        let mut log = Log::InMemory(InMemoryLog::new());
+        let in_memory_log = match log {
             Log::InMemory(ref mut in_memory_log) => in_memory_log,
             _ => panic!("Invalid log type"),
         };
 
-        let collection_uuid_1 =
-            CollectionUuid::from_str("00000000-0000-0000-0000-000000000001").unwrap();
+        let tenant_1 = "tenant_1".to_string();
+        let collection_1 = Collection {
+            collection_id: CollectionUuid::from_str("00000000-0000-0000-0000-000000000001")
+                .unwrap(),
+            name: "collection_1".to_string(),
+            dimension: Some(1),
+            tenant: tenant_1.clone(),
+            database: "database_1".to_string(),
+            ..Default::default()
+        };
+        let collection_uuid_1 = collection_1.collection_id;
+
         in_memory_log.add_log(
             collection_uuid_1,
             InternalLogRecord {
@@ -281,8 +577,18 @@ mod tests {
             },
         );
 
-        let collection_uuid_2 =
-            CollectionUuid::from_str("00000000-0000-0000-0000-000000000002").unwrap();
+        let tenant_2 = "tenant_2".to_string();
+        let collection_2 = Collection {
+            collection_id: CollectionUuid::from_str("00000000-0000-0000-0000-000000000002")
+                .unwrap(),
+            name: "collection_2".to_string(),
+            dimension: Some(1),
+            tenant: tenant_2.clone(),
+            database: "database_2".to_string(),
+            ..Default::default()
+        };
+        let collection_uuid_2 = collection_2.collection_id;
+
         in_memory_log.add_log(
             collection_uuid_2,
             InternalLogRecord {
@@ -303,32 +609,9 @@ mod tests {
             },
         );
 
-        let mut sysdb = Box::new(SysDb::Test(TestSysDb::new()));
+        let mut sysdb = SysDb::Test(TestSysDb::new());
 
-        let tenant_1 = "tenant_1".to_string();
-        let collection_1 = Collection {
-            collection_id: collection_uuid_1,
-            name: "collection_1".to_string(),
-            metadata: None,
-            dimension: Some(1),
-            tenant: tenant_1.clone(),
-            database: "database_1".to_string(),
-            log_position: 0,
-            version: 0,
-        };
-
-        let tenant_2 = "tenant_2".to_string();
-        let collection_2 = Collection {
-            collection_id: collection_uuid_2,
-            name: "collection_2".to_string(),
-            metadata: None,
-            dimension: Some(1),
-            tenant: tenant_2.clone(),
-            database: "database_2".to_string(),
-            log_position: 0,
-            version: 0,
-        };
-        match *sysdb {
+        match sysdb {
             SysDb::Test(ref mut sysdb) => {
                 sysdb.add_collection(collection_1);
                 sysdb.add_collection(collection_2);
@@ -338,23 +621,31 @@ mod tests {
             _ => panic!("Invalid sysdb type"),
         }
 
-        let my_member_id = "1".to_string();
+        let my_member = Member {
+            member_id: "member_1".to_string(),
+            member_ip: "10.0.0.1".to_string(),
+            member_node_name: "node_1".to_string(),
+        };
         let scheduler_policy = Box::new(LasCompactionTimeSchedulerPolicy {});
         let max_concurrent_jobs = 1000;
+        let max_failure_count = 3;
 
         // Set assignment policy
-        let mut assignment_policy = Box::new(RendezvousHashingAssignmentPolicy::new());
-        assignment_policy.set_members(vec![my_member_id.clone()]);
+        let mut assignment_policy = Box::new(RendezvousHashingAssignmentPolicy::default());
+        assignment_policy.set_members(vec![my_member.member_id.clone()]);
 
         let mut scheduler = Scheduler::new(
-            my_member_id.clone(),
+            my_member.member_id.clone(),
             log,
             sysdb.clone(),
+            storage,
             scheduler_policy,
             max_concurrent_jobs,
             1,
             assignment_policy,
             HashSet::new(),
+            3600,              // job_expiry_seconds
+            max_failure_count, // max_failure_count
         );
         // Scheduler does nothing without memberlist
         scheduler.schedule().await;
@@ -369,16 +660,17 @@ mod tests {
         assert_eq!(jobs.count(), 0);
 
         // Set memberlist
-        scheduler.set_memberlist(vec![my_member_id.clone()]);
+        scheduler.set_memberlist(vec![my_member.clone()]);
         scheduler.schedule().await;
         let jobs = scheduler.get_jobs();
         let jobs = jobs.collect::<Vec<&CompactionJob>>();
         // Scheduler ignores collection that failed to fetch last compaction time
         assert_eq!(jobs.len(), 1);
         assert_eq!(jobs[0].collection_id, collection_uuid_1,);
+        scheduler.succeed_collection(collection_uuid_1);
 
         // Add last compaction time for tenant_2
-        match *sysdb {
+        match sysdb {
             SysDb::Test(ref mut sysdb) => {
                 let last_compaction_time_2 = 1;
                 sysdb.add_tenant_last_compaction_time(tenant_2, last_compaction_time_2);
@@ -392,17 +684,20 @@ mod tests {
         assert_eq!(jobs.len(), 2);
         assert_eq!(jobs[0].collection_id, collection_uuid_2,);
         assert_eq!(jobs[1].collection_id, collection_uuid_1,);
+        scheduler.succeed_collection(collection_uuid_1);
+        scheduler.succeed_collection(collection_uuid_2);
 
         // Set disable list.
         std::env::set_var(
             "CHROMA_COMPACTION_SERVICE__COMPACTOR__DISABLED_COLLECTIONS",
-            "[\"00000000-0000-0000-0000-000000000001\"]",
+            format!("[\"{}\"]", collection_uuid_1.0),
         );
         scheduler.schedule().await;
         let jobs = scheduler.get_jobs();
         let jobs = jobs.collect::<Vec<&CompactionJob>>();
         assert_eq!(jobs.len(), 1);
         assert_eq!(jobs[0].collection_id, collection_uuid_2,);
+        scheduler.succeed_collection(collection_uuid_2);
         std::env::set_var(
             "CHROMA_COMPACTION_SERVICE__COMPACTOR__DISABLED_COLLECTIONS",
             "[]",
@@ -410,45 +705,88 @@ mod tests {
         // Even . should work.
         std::env::set_var(
             "CHROMA_COMPACTION_SERVICE.COMPACTOR.DISABLED_COLLECTIONS",
-            "[\"00000000-0000-0000-0000-000000000002\"]",
+            format!("[\"{}\"]", collection_uuid_2.0),
         );
         std::env::set_var(
             "CHROMA_COMPACTION_SERVICE.IRRELEVANT",
-            "[\"00000000-0000-0000-0000-000000000001\"]",
+            format!("[\"{}\"]", collection_uuid_1.0),
         );
         scheduler.schedule().await;
         let jobs = scheduler.get_jobs();
         let jobs = jobs.collect::<Vec<&CompactionJob>>();
         assert_eq!(jobs.len(), 1);
         assert_eq!(jobs[0].collection_id, collection_uuid_1,);
+        scheduler.succeed_collection(collection_uuid_1);
+        scheduler.succeed_collection(collection_uuid_2);
         std::env::set_var(
             "CHROMA_COMPACTION_SERVICE.COMPACTOR.DISABLED_COLLECTIONS",
             "[]",
         );
 
         // Test filter_collections
-        let member_1 = "1".to_string();
-        let member_2 = "5".to_string();
+        let member_1 = Member {
+            member_id: "member_1".to_string(),
+            member_ip: "10.0.0.1".to_string(),
+            member_node_name: "node_1".to_string(),
+        };
+        let member_2 = Member {
+            member_id: "member_2".to_string(),
+            member_ip: "10.0.0.2".to_string(),
+            member_node_name: "node_2".to_string(),
+        };
         let members = vec![member_1.clone(), member_2.clone()];
-        scheduler.set_memberlist(members.clone());
+        scheduler.set_memberlist(members);
         scheduler.schedule().await;
         let jobs = scheduler.get_jobs();
         assert_eq!(jobs.count(), 1);
+        scheduler.succeed_collection(collection_uuid_2);
+
+        let members = vec![member_1.clone()];
+        scheduler.set_memberlist(members);
+        // Test dead jobs
+        std::env::set_var(
+            "CHROMA_COMPACTION_SERVICE.COMPACTOR.DISABLED_COLLECTIONS",
+            "[]",
+        );
+        std::env::set_var("CHROMA_COMPACTION_SERVICE.IRRELEVANT", "[]");
+        for _ in 0..max_failure_count {
+            scheduler.schedule().await;
+            let jobs = scheduler.get_jobs();
+            let jobs = jobs.collect::<Vec<&CompactionJob>>();
+            assert_eq!(jobs.len(), 2);
+            scheduler.fail_collection(collection_uuid_1);
+            scheduler.succeed_collection(collection_uuid_2);
+        }
+        scheduler.schedule().await;
+        let jobs = scheduler.get_jobs();
+        let jobs = jobs.collect::<Vec<&CompactionJob>>();
+        assert_eq!(jobs.len(), 1);
+        assert_eq!(jobs[0].collection_id, collection_uuid_2);
+        scheduler.succeed_collection(collection_uuid_2);
     }
 
     #[tokio::test]
-    #[should_panic(
-        expected = "offset in sysdb is less than offset in log, this should not happen!"
-    )]
-    async fn test_scheduler_panic() {
-        let mut log = Box::new(Log::InMemory(InMemoryLog::new()));
-        let in_memory_log = match *log {
+    #[should_panic(expected = "is less than offset")]
+    async fn test_k8s_integration_scheduler_panic() {
+        let storage = s3_client_for_test_with_new_bucket().await;
+
+        let mut log = Log::InMemory(InMemoryLog::new());
+        let in_memory_log = match log {
             Log::InMemory(ref mut in_memory_log) => in_memory_log,
             _ => panic!("Invalid log type"),
         };
 
-        let collection_uuid_1 =
-            CollectionUuid::from_str("00000000-0000-0000-0000-000000000001").unwrap();
+        let tenant_1 = "tenant_1".to_string();
+        let collection_1 = Collection {
+            name: "collection_1".to_string(),
+            dimension: Some(1),
+            tenant: tenant_1.clone(),
+            database: "database_1".to_string(),
+            ..Default::default()
+        };
+
+        let collection_uuid_1 = collection_1.collection_id;
+
         in_memory_log.add_log(
             collection_uuid_1,
             InternalLogRecord {
@@ -525,23 +863,13 @@ mod tests {
                 },
             },
         );
-        let _ = log.update_collection_log_offset(collection_uuid_1, 2).await;
+        let _ = log
+            .update_collection_log_offset(&tenant_1, collection_uuid_1, 2)
+            .await;
 
-        let mut sysdb = Box::new(SysDb::Test(TestSysDb::new()));
+        let mut sysdb = SysDb::Test(TestSysDb::new());
 
-        let tenant_1 = "tenant_1".to_string();
-        let collection_1 = Collection {
-            collection_id: collection_uuid_1,
-            name: "collection_1".to_string(),
-            metadata: None,
-            dimension: Some(1),
-            tenant: tenant_1.clone(),
-            database: "database_1".to_string(),
-            log_position: 0,
-            version: 0,
-        };
-
-        match *sysdb {
+        match sysdb {
             SysDb::Test(ref mut sysdb) => {
                 sysdb.add_collection(collection_1);
                 let last_compaction_time_1 = 2;
@@ -549,26 +877,34 @@ mod tests {
             }
             _ => panic!("Invalid sysdb type"),
         }
-        let my_ip = "0.0.0.1".to_string();
+        let my_member = Member {
+            member_id: "member_1".to_string(),
+            member_ip: "0.0.0.1".to_string(),
+            member_node_name: "node_1".to_string(),
+        };
         let scheduler_policy = Box::new(LasCompactionTimeSchedulerPolicy {});
         let max_concurrent_jobs = 1000;
+        let max_failure_count = 3;
 
         // Set assignment policy
-        let mut assignment_policy = Box::new(RendezvousHashingAssignmentPolicy::new());
-        assignment_policy.set_members(vec![my_ip.clone()]);
+        let mut assignment_policy = Box::new(RendezvousHashingAssignmentPolicy::default());
+        assignment_policy.set_members(vec![my_member.member_id.clone()]);
 
         let mut scheduler = Scheduler::new(
-            my_ip.clone(),
+            my_member.member_id.clone(),
             log,
             sysdb.clone(),
+            storage,
             scheduler_policy,
             max_concurrent_jobs,
             1,
             assignment_policy,
             HashSet::new(),
+            3600,              // job_expiry_seconds
+            max_failure_count, // max_failure_count
         );
 
-        scheduler.set_memberlist(vec![my_ip.clone()]);
+        scheduler.set_memberlist(vec![my_member.clone()]);
         scheduler.schedule().await;
     }
 }
