@@ -1,51 +1,52 @@
 use async_trait::async_trait;
 use chroma_blockstore::provider::BlockfileProvider;
-use chroma_system::{wrap, ComponentContext, ComponentHandle, Dispatcher, Handler, Orchestrator, TaskMessage, TaskResult};
+use chroma_system::{
+    wrap, ComponentContext, ComponentHandle, Dispatcher, Handler, Orchestrator,
+    OrchestratorContext, TaskMessage, TaskResult,
+};
+use chroma_types::{
+    operator::{Knn, KnnOutput, Merge, RecordMeasure},
+    CollectionAndSegments,
+};
 use tokio::sync::oneshot::Sender;
+use tracing::Span;
 
 use crate::execution::operators::{
-    knn::{KnnOperator, RecordDistance},
-    knn_hnsw::{KnnHnswError, KnnHnswInput, KnnHnswOutput},
-    knn_log::{KnnLogError, KnnLogInput, KnnLogOutput},
-    knn_merge::{KnnMergeError, KnnMergeInput, KnnMergeOperator, KnnMergeOutput},
-    knn_projection::{
-        KnnProjectionError, KnnProjectionInput, KnnProjectionOperator, KnnProjectionOutput,
-    },
-    prefetch_record::{
-        PrefetchRecordError, PrefetchRecordInput, PrefetchRecordOperator, PrefetchRecordOutput,
-    },
+    knn_hnsw::{KnnHnswError, KnnHnswInput},
+    knn_log::{KnnLogError, KnnLogInput},
+    knn_merge::{KnnMergeError, KnnMergeInput, KnnMergeOutput},
 };
 
-use super::knn_filter::{KnnError, KnnFilterOutput, KnnOutput, KnnResult};
+use super::knn_filter::{KnnError, KnnFilterOutput};
 
 /// The `KnnOrchestrator` finds the nearest neighbor of a target embedding given the search domain.
-/// When used together with `KnnFilterOrchestrator`, they evaluate a `<collection>.query(...)` query
+/// When used together with `FilterOrchestrator`, they evaluate a `<collection>.query(...)` query
 /// for the user. We breakdown the evaluation into two parts because a `<collection>.query(...)`
 /// is inherently multiple queries sharing the same filter criteria. Thus we first evaluate
-/// the filter criteria with `KnnFilterOrchestrator`. Then we spawn a `KnnOrchestrator` for each
-/// of the embedding together with a copy of the result from `KnnFilterOrchestrator`, run these
+/// the filter criteria with `FilterOrchestrator`. Then we spawn a `KnnOrchestrator` for each
+/// of the embedding together with a copy of the result from `FilterOrchestrator`, run these
 /// orchestrators in parallel, and join them in the end.
 ///
 ///
 /// # Pipeline
 /// ```text
-///                                                           │                                                               
-///                                                           │                                                               
-///                                                           │                                                               
-///                                                           │                                                               
-///                                                           ▼                                                               
-///                                               ┌───────────────────────┐                                                   
-///                                               │                       │                                                   
-///                                               │ KnnFilterOrchestrator │                                                   
-///                                               │                       │                                                   
-///                                               └───────────┬───────────┘                                                   
-///                                                           │                                                               
-///                                                           │                                                               
-///                                                           │                                                               
-///                        ┌──────────────────────────────────┴─────────────────────────────────────┐                         
-///                        │                                                                        │                         
-///                        │                    ... One branch per embedding ...                    │                         
-///                        │                                                                        │                         
+///                                                           │
+///                                                           │
+///                                                           │
+///                                                           │
+///                                                           ▼
+///                                               ┌───────────────────────┐
+///                                               │                       │
+///                                               │   FilterOrchestrator  │
+///                                               │                       │
+///                                               └───────────┬───────────┘
+///                                                           │
+///                                                           │
+///                                                           │
+///                        ┌──────────────────────────────────┴─────────────────────────────────────┐
+///                        │                                                                        │
+///                        │                    ... One branch per embedding ...                    │
+///                        │                                                                        │
 /// ┌────────────────────  │  ─────────────────────┐                         ┌────────────────────  │  ─────────────────────┐
 /// │                      ▼                       │                         │                      ▼                       │
 /// │               ┌────────────┐ KnnOrchestrator │                         │               ┌────────────┐ KnnOrchestrator │
@@ -76,13 +77,6 @@ use super::knn_filter::{KnnError, KnnFilterOutput, KnnOutput, KnnResult};
 /// │           └──────────┬──────────┘            │                         │           └──────────┬──────────┘            │
 /// │                      │                       │                         │                      │                       │
 /// │                      ▼                       │                         │                      ▼                       │
-/// │         ┌─────────────────────────┐          │                         │         ┌─────────────────────────┐          │
-/// │         │                         │          │                         │         │                         │          │
-/// │         │  KnnProjectionOperator  │          │           ...           │         │  KnnProjectionOperator  │          │
-/// │         │                         │          │                         │         │                         │          │
-/// │         └────────────┬────────────┘          │                         │         └────────────┬────────────┘          │
-/// │                      │                       │                         │                      │                       │
-/// │                      ▼                       │                         │                      ▼                       │
 /// │             ┌──────────────────┐             │                         │             ┌──────────────────┐             │
 /// │             │                  │             │                         │             │                  │             │
 /// │             │  result_channel  │             │           ...           │             │  result_channel  │             │
@@ -90,41 +84,42 @@ use super::knn_filter::{KnnError, KnnFilterOutput, KnnOutput, KnnResult};
 /// │             └────────┬─────────┘             │                         │             └────────┬─────────┘             │
 /// │                      │                       │                         │                      │                       │
 /// └────────────────────  │  ─────────────────────┘                         └────────────────────  │  ─────────────────────┘
-///                        │                                                                        │                         
-///                        │                                                                        │                         
-///                        │                                                                        │                         
-///                        │                           ┌────────────────┐                           │                         
-///                        │                           │                │                           │                         
-///                        └──────────────────────────►│  try_join_all  │◄──────────────────────────┘                         
-///                                                    │                │                                                     
-///                                                    └───────┬────────┘                                                     
-///                                                            │                                                              
-///                                                            │                                                              
-///                                                            ▼                                                              
+///                        │                                                                        │
+///                        │                                                                        │
+///                        │                                                                        │
+///                        │                           ┌────────────────┐                           │
+///                        │                           │                │                           │
+///                        └──────────────────────────►│  try_join_all  │◄──────────────────────────┘
+///                                                    │                │
+///                                                    └───────┬────────┘
+///                                                            │
+///                                                            │
+///                                                            ▼
 /// ```
 #[derive(Debug)]
 pub struct KnnOrchestrator {
     // Orchestrator parameters
+    context: OrchestratorContext,
     blockfile_provider: BlockfileProvider,
-    dispatcher: ComponentHandle<Dispatcher>,
     queue: usize,
 
+    // Collection information
+    collection_and_segments: CollectionAndSegments,
+
     // Output from KnnFilterOrchestrator
     knn_filter_output: KnnFilterOutput,
 
     // Knn operator shared between log and segments
-    knn: KnnOperator,
+    knn: Knn,
 
     // Knn output
-    knn_log_distances: Option<Vec<RecordDistance>>,
-    knn_segment_distances: Option<Vec<RecordDistance>>,
+    batch_distances: Vec<Vec<RecordMeasure>>,
 
-    // Merge and project
-    merge: KnnMergeOperator,
-    knn_projection: KnnProjectionOperator,
+    // Merge
+    merge: Merge,
 
     // Result channel
-    result_channel: Option<Sender<KnnResult>>,
+    result_channel: Option<Sender<Result<Vec<RecordMeasure>, KnnError>>>,
 }
 
 impl KnnOrchestrator {
@@ -132,58 +127,62 @@ impl KnnOrchestrator {
         blockfile_provider: BlockfileProvider,
         dispatcher: ComponentHandle<Dispatcher>,
         queue: usize,
+        collection_and_segments: CollectionAndSegments,
         knn_filter_output: KnnFilterOutput,
-        knn: KnnOperator,
-        knn_projection: KnnProjectionOperator,
+        knn: Knn,
     ) -> Self {
         let fetch = knn.fetch;
-        let knn_segment_distances = if knn_filter_output.hnsw_reader.is_none() {
-            Some(Vec::new())
+        let batch_distances = if knn_filter_output.hnsw_reader.is_none() {
+            vec![Vec::new()]
         } else {
-            None
+            Vec::new()
         };
+        let context = OrchestratorContext::new(dispatcher);
         Self {
+            context,
             blockfile_provider,
-            dispatcher,
             queue,
+            collection_and_segments,
             knn_filter_output,
             knn,
-            knn_log_distances: None,
-            knn_segment_distances,
-            merge: KnnMergeOperator { fetch },
-            knn_projection,
+            batch_distances,
+            merge: Merge { k: fetch },
             result_channel: None,
         }
     }
 
     async fn try_start_knn_merge_operator(&mut self, ctx: &ComponentContext<Self>) {
-        if let (Some(log_distances), Some(segment_distances)) = (
-            self.knn_log_distances.as_ref(),
-            self.knn_segment_distances.as_ref(),
-        ) {
+        if self.batch_distances.len() == 2 {
             let task = wrap(
                 Box::new(self.merge.clone()),
                 KnnMergeInput {
-                    first_distances: log_distances.clone(),
-                    second_distances: segment_distances.clone(),
+                    batch_measures: self.batch_distances.drain(..).collect(),
                 },
                 ctx.receiver(),
+                self.context.task_cancellation_token.clone(),
             );
-            self.send(task, ctx).await;
+            self.send(task, ctx, Some(Span::current())).await;
         }
     }
 }
 
 #[async_trait]
 impl Orchestrator for KnnOrchestrator {
-    type Output = KnnOutput;
+    type Output = Vec<RecordMeasure>;
     type Error = KnnError;
 
     fn dispatcher(&self) -> ComponentHandle<Dispatcher> {
-        self.dispatcher.clone()
+        self.context.dispatcher.clone()
     }
 
-    fn initial_tasks(&self, ctx: &ComponentContext<Self>) -> Vec<TaskMessage> {
+    fn context(&self) -> &OrchestratorContext {
+        &self.context
+    }
+
+    async fn initial_tasks(
+        &mut self,
+        ctx: &ComponentContext<Self>,
+    ) -> Vec<(TaskMessage, Option<Span>)> {
         let mut tasks = Vec::new();
 
         let knn_log_task = wrap(
@@ -191,13 +190,14 @@ impl Orchestrator for KnnOrchestrator {
             KnnLogInput {
                 logs: self.knn_filter_output.logs.clone(),
                 blockfile_provider: self.blockfile_provider.clone(),
-                record_segment: self.knn_filter_output.record_segment.clone(),
+                record_segment: self.collection_and_segments.record_segment.clone(),
                 log_offset_ids: self.knn_filter_output.filter_output.log_offset_ids.clone(),
                 distance_function: self.knn_filter_output.distance_function.clone(),
             },
             ctx.receiver(),
+            self.context.task_cancellation_token.clone(),
         );
-        tasks.push(knn_log_task);
+        tasks.push((knn_log_task, Some(Span::current())));
 
         if let Some(hnsw_reader) = self.knn_filter_output.hnsw_reader.as_ref().cloned() {
             let knn_segment_task = wrap(
@@ -212,8 +212,9 @@ impl Orchestrator for KnnOrchestrator {
                     distance_function: self.knn_filter_output.distance_function.clone(),
                 },
                 ctx.receiver(),
+                self.context.task_cancellation_token.clone(),
             );
-            tasks.push(knn_segment_task);
+            tasks.push((knn_segment_task, Some(Span::current())));
         }
 
         tasks
@@ -223,49 +224,47 @@ impl Orchestrator for KnnOrchestrator {
         self.queue
     }
 
-    fn set_result_channel(&mut self, sender: Sender<KnnResult>) {
+    fn set_result_channel(&mut self, sender: Sender<Result<Vec<RecordMeasure>, KnnError>>) {
         self.result_channel = Some(sender)
     }
 
-    fn take_result_channel(&mut self) -> Sender<KnnResult> {
-        self.result_channel
-            .take()
-            .expect("The result channel should be set before take")
+    fn take_result_channel(&mut self) -> Option<Sender<Result<Vec<RecordMeasure>, KnnError>>> {
+        self.result_channel.take()
     }
 }
 
 #[async_trait]
-impl Handler<TaskResult<KnnLogOutput, KnnLogError>> for KnnOrchestrator {
+impl Handler<TaskResult<KnnOutput, KnnLogError>> for KnnOrchestrator {
     type Result = ();
 
     async fn handle(
         &mut self,
-        message: TaskResult<KnnLogOutput, KnnLogError>,
+        message: TaskResult<KnnOutput, KnnLogError>,
         ctx: &ComponentContext<Self>,
     ) {
-        let output = match self.ok_or_terminate(message.into_inner(), ctx) {
+        let output = match self.ok_or_terminate(message.into_inner(), ctx).await {
             Some(output) => output,
             None => return,
         };
-        self.knn_log_distances = Some(output.record_distances);
+        self.batch_distances.push(output.distances);
         self.try_start_knn_merge_operator(ctx).await;
     }
 }
 
 #[async_trait]
-impl Handler<TaskResult<KnnHnswOutput, KnnHnswError>> for KnnOrchestrator {
+impl Handler<TaskResult<KnnOutput, KnnHnswError>> for KnnOrchestrator {
     type Result = ();
 
     async fn handle(
         &mut self,
-        message: TaskResult<KnnHnswOutput, KnnHnswError>,
+        message: TaskResult<KnnOutput, KnnHnswError>,
         ctx: &ComponentContext<Self>,
     ) {
-        let output = match self.ok_or_terminate(message.into_inner(), ctx) {
+        let output = match self.ok_or_terminate(message.into_inner(), ctx).await {
             Some(output) => output,
             None => return,
         };
-        self.knn_segment_distances = Some(output.record_distances);
+        self.batch_distances.push(output.distances);
         self.try_start_knn_merge_operator(ctx).await;
     }
 }
@@ -279,63 +278,11 @@ impl Handler<TaskResult<KnnMergeOutput, KnnMergeError>> for KnnOrchestrator {
         message: TaskResult<KnnMergeOutput, KnnMergeError>,
         ctx: &ComponentContext<Self>,
     ) {
-        let output = message
-            .into_inner()
-            .expect("KnnMergeOperator should not fail");
+        let output = match self.ok_or_terminate(message.into_inner(), ctx).await {
+            Some(output) => output,
+            None => return,
+        };
 
-        // Prefetch records before projection
-        let prefetch_task = wrap(
-            Box::new(PrefetchRecordOperator {}),
-            PrefetchRecordInput {
-                logs: self.knn_filter_output.logs.clone(),
-                blockfile_provider: self.blockfile_provider.clone(),
-                record_segment: self.knn_filter_output.record_segment.clone(),
-                offset_ids: output
-                    .record_distances
-                    .iter()
-                    .map(|record| record.offset_id)
-                    .collect(),
-            },
-            ctx.receiver(),
-        );
-        self.send(prefetch_task, ctx).await;
-
-        let projection_task = wrap(
-            Box::new(self.knn_projection.clone()),
-            KnnProjectionInput {
-                logs: self.knn_filter_output.logs.clone(),
-                blockfile_provider: self.blockfile_provider.clone(),
-                record_segment: self.knn_filter_output.record_segment.clone(),
-                record_distances: output.record_distances,
-            },
-            ctx.receiver(),
-        );
-        self.send(projection_task, ctx).await;
-    }
-}
-
-#[async_trait]
-impl Handler<TaskResult<PrefetchRecordOutput, PrefetchRecordError>> for KnnOrchestrator {
-    type Result = ();
-
-    async fn handle(
-        &mut self,
-        _message: TaskResult<PrefetchRecordOutput, PrefetchRecordError>,
-        _ctx: &ComponentContext<Self>,
-    ) {
-        // The output and error from `PrefetchRecordOperator` are ignored
-    }
-}
-
-#[async_trait]
-impl Handler<TaskResult<KnnProjectionOutput, KnnProjectionError>> for KnnOrchestrator {
-    type Result = ();
-
-    async fn handle(
-        &mut self,
-        message: TaskResult<KnnProjectionOutput, KnnProjectionError>,
-        ctx: &ComponentContext<Self>,
-    ) {
-        self.terminate_with_result(message.into_inner().map_err(|e| e.into()), ctx);
+        self.terminate_with_result(Ok(output.measures), ctx).await;
     }
 }
